
#!/bin/sh
#  Script to generate a set of Policy certificates, UseCondition an
#  Attribute certificates from the xml certificates in the LBL resource tree.



echo "\nRemoving any previously generated certificates"
rm -f certs/*.0
rm -f certs/*.1
rm -f TOP/*.0
rm -f TOP/test1/*.0
rm -f TOP/cgi-bin/extauth/*.0
rm -f TOP/cgi-bin/grouptest/*.0
rm -f idCerts/*.0

# Generate the root .htauthority file from LBL/rootPolicyCert.xml, signed by
# akentitest.pem.

echo "\nGenerating TOP/.htauthority from TOP/rootPolicy.xml"
rm -f TOP/.htauthority
bin/akenti CertGen  -k idCerts/akentitest.p12 -p bin/pfile -o TOP/.htauthority -i TOP/rootPolicy.xml

#Generate a use Condition for the root from LBL/uc.xml and store it in 
# certs/LBLuc.104b8965.0. Sign it by akentiTesting

echo "\nGenerating certs/TOPuc from TOP/uc.xml"
rm -f certs/TOPuc.*
bin/akenti CertGen  -k idCerts/akentitest.p12 -p bin/pfile -o certs/TOPuc -i TOP/uc.xml


#Generate a .htauthority for TOP/cgi-bin/grouptest from TOP/cgi-bin/grouptest/policy.xml signed by 
#  StakeHolder I
echo "\nGenerating TOP/cgi-bin/grouptest/.htauthority from TOP/cgi-bin/grouptest/policy.xml"
rm -f TOP/cgi-bin/grouptest/.htauthority
bin/akenti CertGen  -k idCerts/issuer.p12 -p bin/pfile -o TOP/cgi-bin/grouptest/.htauthority -i TOP/cgi-bin/grouptest/policy.xml

#Generate the uc for TOP/cgi-bin/grouptest signed by Akenti Testing and stored in certs
echo "\nGenerating certs/grouptestuc from TOP/cgi-bin/grouptest/uc1.xml"
rm -f  certs/grouptestuc.* 
bin/akenti CertGen  -k idCerts/akentitest.p12 -p bin/pfile -o certs/grouptestuc -i TOP/cgi-bin/grouptest/uc1.xml

#Generate a .htauthority for TOP/cgi-bin/extauth from TOP/cgi-bin/extauth/policy.xml signed by 
#  StakeHolder I
echo "\nGenerating TOP/cgi-bin/extauth/.htauthority from TOP/cgi-bin/extauth/policy.xml"
rm -f TOP/cgi-bin/extauth/.htauthority
bin/akenti CertGen  -k idCerts/issuer.p12 -p bin/pfile -o TOP/cgi-bin/extauth/.htauthority -i TOP/cgi-bin/extauth/policy.xml

#Generate the uc for TOP/cgi-bin/extauth signed by Akenti Testing and stored in certs
echo "\nGenerating certs/extauthuc from TOP/cgi-bin/extauth/uc2.xml"
rm -f  certs/extauthuc.* 
bin/akenti CertGen  -k idCerts/issuer.p12 -p bin/pfile -o certs/extauthuc -i TOP/cgi-bin/extauth/uc2.xml


# Generate an attribute certficate granting StakeHolder I the role of tester
# signed by Akenti Testing.
echo "\nGenerating certs/atRoleTester from certs/atRoleTester.xml"
rm -f certs/atRoleTester.50a64776.*
bin/akenti CertGen  -k idCerts/akentitest.p12 -p bin/pfile -o certs/atRoleTester -i certs/atRoleTester.xml

# Add the hash names for the identity certificates we are using
set `bin/akenti HashX509 -x idCerts/akentitest.pem`; linkname=${4}.0
echo "linking ..idCerts/AkTest.pem by its hash name: $linkname"
ln -s akentitest.pem idCerts/$linkname
set `bin/akenti HashX509 -x idCerts/issuer.pem`; linkname=${4}.0
echo "linking ..idCerts/issuer.pem by its hash name: $linkname"
ln -s issuer.pem idCerts/$linkname

# Verify all the certs
echo "\nVerifying TOP/.htauthority"
bin/akenti VerifyAkentiCert -a TOP/.htauthority -i idCerts/akentitest.pem

echo "\nVerifying certs/TOPuc"
bin/akenti VerifyAkentiCert -a certs/TOPuc.*.0 -i idCerts/akentitest.pem

echo "\nVerifying TOP/cgi-bin/extauth/.htauthority"
bin/akenti VerifyAkentiCert -a TOP/cgi-bin/extauth/.htauthority -i idCerts/issuer.pem

echo "\nVerifying certs/extauthuc"
bin/akenti VerifyAkentiCert -a certs/extauthuc.*.0 -i idCerts/issuer.pem

echo "\nVerifying TOP/cgi-bin/grouptest/.htauthority"
bin/akenti VerifyAkentiCert -a TOP/cgi-bin/grouptest/.htauthority -i idCerts/issuer.pem

echo "\nVerifying certs/grouptestuc"
bin/akenti VerifyAkentiCert -a certs/grouptestuc.*.0 -i idCerts/akentitest.pem

echo "\nVerifying certs/atRoleTester"
bin/akenti VerifyAkentiCert -a certs/atRoleTester.*.0 -i idCerts/akentitest.pem
