#include <stdio.h>



static char* usage[] = {
   "usage: akentiClient args(in the order specified)\n",
   " -host 	- Server to be contacted [localhost] \n",
   " -port	- Port at which the Server listens [8443] \n",
   " -cert	- Subject Certificate [./client.cert] (optional) \n",
   " -key	- Subject Key [./client.key] (optional) \n",
   " -cafile	- CA Certificate that client accepts [./ca.pem] (optional) \n",
   " -capath	- path of CA Certificates that client accepts [./] (optional) \n",
   " -resource	- Resource to be accessed\n",
   " -help	- general help\n",
   NULL
};

static void printUsage() {
  int count=0;

  while (usage[count] != NULL) {
    printf(usage[count++]);
  }
}

static void exitError(char *message) {
  printf("ERROR : %s\n",  message);
  printUsage();
  exit(-1);
}

static void fatalError(char *message) {
  printf("ERROR : %s\n", message);
  exit(-1);
}


int main(int argc, char **argv) {

  char *host = "localhost";
  int port = 8443;
  char *cacert = "./ca.pem";
  char *capath = "./";
  char *clientkey ="./client.key";
  char *clientcert="./client.cert";
  char *resource;
 
  argc--;
  argv++;

  if (argc == 0) {
    exitError("No arguments");
  }

  while (argc >= 1) {
    if (strcmp(*argv, "-host") == 0) {
      if (--argc < 1) {
        exitError("Missing host");
      }
      host = *(++argv);
      argv++;
      argc--;
    } else if (strcmp(*argv, "-port") == 0) {
      if (--argc < 1) {
        exitError("Missing port");
      }
      port = atoi(*(++argv));

      if (port == 0) {
        exitError("Invalid Port ");
      }

      argv++;
      argc--;
    } else if (strcmp(*argv, "-cert") == 0) {
      if (--argc < 1) {
        exitError("Missing client certificate");
      }
      clientcert = *(++argv);
      argv++;
      argc--;
    } else if (strcmp(*argv, "-key") == 0) {
      if (--argc < 1) {
        exitError("Missing client key");
      }
      clientkey = *(++argv);
      argv++;
      argc--;
    } else if (strcmp(*argv, "-cafile") == 0) {
       if (--argc < 1) {
        exitError("Missing Certificate Authority certificate");
      }
      cacert = *(++argv);
      argv++;
      argc--;
    } else if (strcmp(*argv, "-capath") == 0) {
       if (--argc < 1) {
        exitError("Missing Certificate Authority certificate");
       }
       capath = *(++argv);
       argv++;
       argc--;
    } else if (strcmp(*(++argv), "-resource") == 0) {
       if (--argc < 1) {
	  exitError("Missing resource");
       }
       resource = *(++argv);
       argv++;
       argc--;
    } else if (strcmp(*argv, "-help") == 0 || strcmp(*argv, "-h") == 0) {
      printUsage();
      exit(-1);
    } else {
      exitError("Invalid argument");
    } 
  }
  CheckAccess( host, port, clientkey, clientcert, cacert, capath, resource);
} 

 
