#include "AkentiCertificateFactory.hpp"
#include "AttributeCertificateImpl.hpp"
#include "PolicyCertificateImpl.hpp"
#include "UseCondCertificateImpl.hpp"
#include "CapabilityCertificateImpl.hpp"
#include "AkentiX509CertificateImpl.hpp"
#include "CacheCertificateImpl.hpp"
#include "Exception.hpp"
#include "Error.hpp"

namespace Akenti {

typedef AkentiCertificateFactory ACF;

ACF ACF::fac = AkentiCertificateFactory();

ACF::AkentiCertificateFactory() {
  addPrototype(new UseCondCertificateImpl);
  addPrototype(new AttributeCertificateImpl);
  addPrototype(new PolicyCertificateImpl);
  addPrototype(new CapabilityCertificateImpl);
  addPrototype(new AkentiX509CertificateImpl);
  addPrototype(new CacheCertificateImpl);
}

ACF::~AkentiCertificateFactory() {
  map<string, AkentiCertificateImpl*>::iterator pos;

  for (pos = prototypes.begin(); pos != prototypes.end(); ++pos) {
    delete(pos->second);
  }
}

void
ACF::addPrototype(AkentiCertificateImpl* impl) {
  if (!impl) {
    throw Error("AkentiCertificateFactory:addPrototype:illegal argument");
  }

  string type = impl->getType();

  if (prototypes[type]) {
    delete(prototypes[type]);
  }

  prototypes[type] = impl;
}

Akenti::AkentiCertificateImpl*
ACF::getPrototype(const string& type) {
  if (!prototypes[type]) {
    throw Exception("AkentiCertificateFactory:getPrototype:type not supported");
  }

  AkentiCertificateImpl* impl = prototypes[type]->newInstance();

  if (!impl) {
    throw Error("AkentiCertificateFactory:getPrototype:bad alloc");
  }

  return impl;
}

Akenti::AkentiCertificateFactory& 
ACF::getAkentiCertificateFactory() {
  return fac;
}

}
