#ifndef ATTRIBUTE_CERT_GENERATOR_HPP
#define ATTRIBUTE_CERT_GENERATOR_HPP

#include "AkentiCertGenerator.hpp"

namespace Akenti {

class Condition;

/**
  * Used to generate an <i>Attribute</i> certificate.
  *
  * <PRE>
  * <b>Usage:</b>
  *   AttributeCertGenerator gen;
  *
  *   // first call all the set methods in <i>AkentiCertGenerator</i>
  *   gen.setPrincipal(somePrincipal);
  *   gen.setAttribute("GROUP");
  *   gen.setValue("DSD");
  *   // gen.setCondition(someCondition);  <i>optional</i>
  *   // finally call the sign method in  <i>AkentiCertGenerator</i>    
  * </PRE>
  *
  * @version 1.1 00/05/25
  * @see AkentiCertGenerator
  * @see AkentiPrincipal 
  * @see AttributeCertificateImpl 
  * @see AkentiCertificate 
  * 
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari 
  **/

class AttributeCertGenerator : public AkentiCertGenerator {

public:

  /**
    * Constructs this object.
    */
  AttributeCertGenerator();

  /**
    * Destroys this object.
    */
  ~AttributeCertGenerator();
  
  /**
    * Sets the principal for the Akenti Certificate.
    *
    * @param principal the principal of the Akenti Certificate
    */

  void setPrincipal(const AkentiPrincipal& principal);

  /**
    * Sets the <i>attribute</i> component of the attribute/value pair
    * that the newly generated certificate will attest to.
    *
    * @see setValue(const string&)
    * @see AttributeCertificateImpl 
    */
  void setAttribute(const string& attribute);

  /**
    * Sets the <i>value</i> component of the attribute/value pair
    * that the newly generated certificate will attest to.
    *
    * @see setAttribute(const string&)
    * @see AttributeCertificateImpl 
    */
  void setValue(const string& value);

  /**
    * Sets the Condition that will need to be satisfied
    * before the attribute/value pair are attested to by
    * the newly genarated certificate.
    *
    * Note: The condition is optional.
    *
    * @see AttributeCertificateImpl 
    */
  void setCondition(const Condition& condition);

};

} // end of name space

#endif
