#include "AkentiOutputStream.hpp"
#include "StringTokenizer.hpp"
#include "AkentiObject.hpp"
#include "DistinguishedName.hpp"
#include "UtcTime.hpp"
#include "URL.hpp"
#include "IOException.hpp"

namespace Akenti {

typedef AkentiOutputStream AOS;

AOS::AkentiOutputStream(ostream& os) : os(os) {
}


AkentiOutputStream& 
AOS::write(const char *ptr, int n) {
  if (!os.write(ptr, n)) {
    throw IOException("AkentiOutputStream:write:failed");
  }
  
  return *this;
}

AkentiOutputStream& 
AOS::operator<<(char c) {
  if (!(os << c)) {
    throw IOException("AkentiOutputStream:writing char failed");
  }

  return *this;
}

AkentiOutputStream&
AOS::operator<<(int n) {
  if (!(os << n)) {
    throw IOException("AkentiOutputStream:writing int failed");
  }

  return *this;
}

AkentiOutputStream& 
AOS::operator<<(bool b) {
  if (!(os << b)) {
    throw IOException("AkentiOutputStream:writing bool failed");
  }

  return *this;
}

AkentiOutputStream& 
AOS::operator<<(long l) {
  if (!(os << l)) {
    throw IOException("AkentiOutputStream:writing long failed");
  }

  return *this;
}

AkentiOutputStream& 
AOS::operator<<(double d) {
  if (!(os << d)) {
    throw IOException("AkentiOutputStream:writing double failed");
  }

  return *this;
}

AkentiOutputStream& 
AOS::operator<<(const string& s) {
  StringTokenizer st(s, " ", false);
  os << st.nextToken();

  while (st.hasMoreTokens()) {
    os << "\\ " << st.nextToken();
  }

  return *this;
}

AkentiOutputStream& 
AOS::operator<<(const UtcTime& utcTime) {
  if (!(os << utcTime)) {
    throw IOException("AkentiOutputStream:writing UtcTime failed");
  }

  return *this;
}
  
AkentiOutputStream& 
AOS::operator<<(const URL& url) {
  operator<<(url.toString());
  return *this;
}

AkentiOutputStream& 
AOS::operator<<(const DistinguishedName& dn) {
  operator<<(dn.toString());
  return *this;
}

AkentiOutputStream& 
AOS::operator<<(const AkentiObject& obj) {
  obj.writeObject(*this);
  return *this;
}

}
