#include <sstream> 
#include <iostream> 
#include "CertificateRetrievalImpl.hpp" 
#include "AkentiCertificate.hpp"
#include "CertificateFactory.hpp"
#include "Certificate.hpp"
#include "URL.hpp"
#include "Exception.hpp"
#include "Logger.hpp"

using namespace Akenti;
using namespace std;

typedef CertificateRetrievalImpl CRI;

CRI::CertificateRetrievalImpl() {
}

CRI::~CertificateRetrievalImpl() {
}

void
CRI::getCertificates(const URL& url, 
                     map<string, string>& args, 
                     vector<Certificate>& certs) const {
  vector<string> results;
  string enc = args["enc"];;
 
  try {
    getContent(url, args, results, enc);
  } catch(Exception& ex) {
    Logger::write(LOG_L5,"CERT_RETRIEV" , ex.getMessage());
    return;
  }

  ostringstream os;

  os << results.size();

  Logger::write(LOG_L5, "CERT_RETRIEV", "SIZE=" + os.str());

  CertificateFactory fac = CertificateFactory::getInstance();

  if (enc.size()) {

    for (int i = 0;  i < results.size(); ++i) {
      try {
        Certificate temp = fac->initCertificate(enc, results[i]);

        certs.push_back(temp);
      } catch(Exception& ex) {
        Logger::write(LOG_L5,"CERT_RETRIEV" , ex.getMessage());
      }
    }

    return;
  }

  for (int i = 0;  i < results.size(); ++i) {
    enc = (results[i].find("-----BEGIN") != -1? "PEM" : "DER");

    try {
      Certificate temp = fac->initCertificate(enc, results[i]);

      certs.push_back(temp);
    } catch(Exception& ex) {
      Logger::write(LOG_L5,"CERT_RETRIEV" , ex.getMessage());
    }
  }
}

void
CRI::getAkentiCertificates(const URL& url, 
                           map<string, string>& args, 
                           vector<AkentiCertificate>& certs) const {
  string temp = url.toString();

  vector<string> results;
  string enc = args["enc"];

  try {
    getContent(url, args, results, enc);
  } catch(Exception& ex) {
    Logger::write(LOG_L5,"AK_CERT_RETRIEV", ex.getMessage());
    return;
  }

  ostringstream os;
  os << results.size();

  Logger::write(LOG_L5, "AK_CERT_RETRIEV", "SIZE=" + os.str());

  if (enc.size()) {
    for (int i = 0; i < results.size(); ++i) {
      try {
        istringstream is(results[i]);

        AkentiCertificate cert = AkentiCertificate::initFromStream(enc, is);

        if (cert->getType() == args["type"]) {
          certs.push_back(cert);
        }
      } catch(Exception& ex) {
        Logger::write(LOG_L5,"AK_CERT_RETRIEV", ex.getMessage());
      }
    }

    return;
  }

  for (int i = 0;  i < results.size(); ++i) {
    enc = (results[i].find("-----BEGIN") != -1? "PEM" : "XML");

    try {
      istringstream is(results[i]);

      AkentiCertificate cert = AkentiCertificate::initFromStream(enc, is);

      if (cert->getType() == args["type"]) {
        certs.push_back(cert);
      }
    } catch(Exception& ex) {
      Logger::write(LOG_L5,"AK_CERT_RETRIEV", ex.getMessage());
    }
  }
}
