#include "FileCertificateRetrievalImpl.hpp"
#include "URLConnection.hpp"
#include "StringUtil.hpp"
#include "Exception.hpp"
#include "File.hpp"
#include "URL.hpp"
#include <sstream>

using std::istringstream;

namespace Akenti {

typedef FileCertificateRetrievalImpl FCRI;

FCRI::FileCertificateRetrievalImpl() {
}

FCRI::~FileCertificateRetrievalImpl() {
}

  class HashFilter : public Filter {

    public:

      HashFilter(const string& hash) : hash(hash) {
      }

      bool 
      accept(const string& name) const {
        int idx = name.find(hash);

        if (idx == string::npos) {
          return false;
        } else if (idx != 0 && name[idx - 1] != '.') {
          return false;
        }

        int temp = name.length();

        if ((idx += hash.length()) == temp) {
          return true;
        } else if (name[idx++] != '.') {
          return false;
        }

        istringstream is(name.substr(idx, temp - idx));

        return is >> temp;
      }

    private:

      string hash;
   };

void
FCRI::getContent(const URL& url, 
                 map<string, string>& args, 
                 vector<string>& results,
                 string& enc) const {
  HashFilter filter(args["hash"]);

  getContent(url, filter, results);
}
 
void
FCRI::getContent(const URL& url, const Filter& filter, vector<string>& results) const {
  File file(url.getFile());

  if (!file.exists()) {
    throw Exception("FileCertificateRetrieval:getContent:file does not exist");
  } else if (!file.isDirectory()) {
    if (!filter.accept(file.getName())) {
      throw Exception("FileCertificateRetrieval:getContent:file did not match");
    }
    // URL is not a directory and the file name matches the filter
    URLConnection conn = url.getURLConnection();

    conn->connect();
    results.push_back(conn->getContent());
    conn->close();
    return;
  } 

  vector<string> fileNames = file.list();

  for (int i = 0; i < fileNames.size(); ++i) {
    string name = url.getFile() + '/' + fileNames[i];
  
    if (!File(name).isDirectory() && filter.accept(fileNames[i])) {
      URL temp("file", "", -1, name);
      URLConnection conn = temp.getURLConnection();

      conn->connect();
      results.push_back(conn->getContent());
      conn->close();
    }
  }
}

}
