#include "HttpCertificateRetrievalImpl.hpp"
#include "URL.hpp"
#include "URLConnection.hpp"
#include "Filter.hpp"
#include "StringTokenizer.hpp"
#include "Exception.hpp"

using namespace Akenti;

typedef HttpCertificateRetrievalImpl HCRI;

HCRI::HttpCertificateRetrievalImpl() {
}

HCRI::~HttpCertificateRetrievalImpl() {
}

static string 
removeHttpHeader(const string& content) {
  int index = content.find("Content-Type");

  index = content.find("\n", index);
  return content.substr(index + 3);      
}

void
HCRI::getContent(const URL& url, const Filter& filter, vector<string>& results) const {
  URLConnection conn = url.getURLConnection();

  conn->connect();

  string content = conn->getContent();
  
  conn->close();

  // code = The 9th character of content
  // Example: "HTTP/1.1 400...."
  char code = content[9];   

  switch(code) {
    case '2':
      // do nothing, continue
      break;
    case '3':
      getContent(URL(url.toString() + '/'), filter, results);
      return;
    case '4':
      throw Exception("HttpCertificateRetrieval:getContent:file not found");
    default:
      throw Exception("HttpCertificateRetrieval:getContent:unknown code");
  }

  string temp = url.toString();

  if (temp[temp.length() -1] != '/') {   // file
    string file = url.getFile();
    string filename = file.substr(file.find_last_of("/") + 1);

    if (!filter.accept(filename)) {
      throw Exception("HttpCertificateRetrieval:getContent:file did not match");
    }

    results.push_back(removeHttpHeader(content)); 
    return;
  }                                     // directory

  StringTokenizer tokenizer(content, "[]", false);

  tokenizer.nextToken();

  while (tokenizer.hasMoreTokens()) {
    string type = tokenizer.nextToken();
    string token = tokenizer.nextToken();
    
    if (type != "DIR") {
      StringTokenizer tokenizer(token, "\"", false);

      tokenizer.nextToken();

      string filename = tokenizer.nextToken();

      if (filter.accept(filename)) {
          URL temp(url.toString() + filename);
          URLConnection conn = temp.getURLConnection();

          conn->connect();
          string content = conn->getContent();

          results.push_back(removeHttpHeader(content));
          conn->close();
      }
    }
  }
}
