#ifndef LDAP_CERTIFICATE_RETRIEVAL_IMPL_HPP
#define LDAP_CERTIFICATE_RETRIEVAL_IMPL_HPP

#include "CertificateRetrievalImpl.hpp"

using namespace Akenti;

namespace Akenti {

/**
  * Class to retrieve certificate from a LDAP server
  *
  * @see CertificateRetrievalImpl
  * @see URL
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

class LdapCertificateRetrievalImpl : public CertificateRetrievalImpl {

public:

  /**
    * Constructor
    */
  LdapCertificateRetrievalImpl();

  /**
   * Destructor
    */
  ~LdapCertificateRetrievalImpl();

  /**
   * Calls the openLdap library routines to get the X509
   * certificate(s) for any entries with the given base and filter.
   * This is expected to be called with a base and filter that 
   * uniquely identifies one user, although that user might have
   * multiple certificates.
   *
   * @param url - host and optional port of the ldap server
   * @param args - a map of the following arguments
   *              "type" must be X509
   *              "base" the ldap search base
   *              "filter" the ldap search filter, defaults to "cn=*"
   *              "scope" one of "onelevel", "subtree", or defaults to BASE
   * @param results - a returned vector of PEM/DER encoded X509 certificates
   * @param enc - a return value for the type of certificate encoding that is
   *              used. Currently we assume DER.
   *
   */
  void
  getContent(const URL& url, 
             map<string, string>& args,
             vector<string>& results,
             string& enc) const;

};

} // end of name space

#endif
