#include "FileExternalCacheImpl.hpp"
#include "AkentiCertificate.hpp"
#include "IOException.hpp"
#include "Exception.hpp"
#include "Properties.hpp"
#include "File.hpp"
#include "AkentiConstants.hpp"

using namespace Akenti;

typedef FileExternalCacheImpl FECI;

FECI::FileExternalCacheImpl() : cacheDirectory(createCacheDirectory()) {
}

FECI::~FileExternalCacheImpl() {
}

string 
FECI::createCacheDirectory() {
  Properties& props = Properties::getSystemProperties();
  string cacheDirectory = props.getValue("CacheDir", CacheDir);

  File file(cacheDirectory);

  if (file.exists() && !file.isDirectory()) {
    throw IOException(
      "FileExternalCacheImpl:createCacheDirectory:unable to create " + file.getAbsolutePath() 
    );
  } else if (!file.exists()) {
    file.mkdir();
  }

  if (cacheDirectory[cacheDirectory.length() - 1] != '/') {
    cacheDirectory += '/';
  }

  return cacheDirectory;
}

string 
FECI::createResourceDirectory(const string& resource) {
  string dir = cacheDirectory;
 
  dir += resource;
  
  File file(dir);

  if (!file.exists()) {
    file.mkdir();
  } else if (!file.isDirectory()) {
    throw IOException(
      "FileExternalCacheImpl:constructor:unable to crete directory"
    );
  }
  
  dir += '/';
  return dir;
}

bool 
FECI::openConnection() {
  return true;
} 

bool
FECI::storeCacheCertificate(const string& rootResourceName, 
                            const CacheCertificate& cacheCert,
                            const string& dirName) {
  try {
    AkentiCertificate::writeToFile(
      "PEM",
      createResourceDirectory(rootResourceName) + dirName + computeHash(cacheCert->hashCode()), 
      cacheCert 
    );
    
    return true;
  } catch(Exception& ex) {
    // LOG THIS ?
    return false;
  }
}

CacheCertificate
FECI::retrieveCacheCertificate(const string& rootResourceName, 
                               const string& key,
                               const string& dirName) {

  try {
    AkentiCertificate cert  = AkentiCertificate::initFromFile(
      "PEM",
      createResourceDirectory(rootResourceName) + dirName + computeHash(key)
    );
    
    return (CacheCertificate&) cert;
  } catch(Exception& ex) {
    // LOG THIS ?
  } catch(Error& ex) {
    // LOG THIS ?
  }

  return CacheCertificate();
}

bool 
FECI::flushCacheCertificates(const string& rootResourceName,
                             const string& key,
                             const string& dirName) {
  return false;
}

bool 
FECI::closeConnection() { 
  return true;
}

