#include <sstream>
#include <map>
#include "ServerExternalCacheImpl.hpp"
#include "AkentiCertificate.hpp"
#include "Socket.hpp"
#include "CacheRequest.hpp"
#include "CacheResponse.hpp"
#include "IOException.hpp"
#include "Exception.hpp"
#include "Properties.hpp"
#include "AkentiConstants.hpp"

using std::istringstream;
using std::map;

typedef ServerExternalCacheImpl SECI;

namespace Akenti {

static map<int, Socket> conns;

static int
getThreadID() {
  return 0;
}

}

SECI::ServerExternalCacheImpl() {
}

SECI::~ServerExternalCacheImpl() {
}

bool 
SECI::openConnection() {
  Properties& props = Properties::getSystemProperties();
  string host = props.getValue("CacheHost", CacheHost);
  istringstream is(props.getValue("CachePort", CachePort));
   
  int port;

  if (!(is >> port) || port <= 0) {
    return false;
  }
  
  int threadID = Akenti::getThreadID();

  if (conns.find(threadID) != conns.end()) {
    return true;
  }

  try {
    Socket sock(host, port);
    
    sock.connect();
    conns[threadID] = sock;
    return true;
  } catch(IOException& ex) {
    return false;
  }
}

bool
SECI::storeCacheCertificate(const string& rootResourceName, 
                            const CacheCertificate& caCert,
                            const string& dirName) {
  try {
    Socket sock = conns[Akenti::getThreadID()];
    CacheRequest req("UPDATE");
   
    req.setRootResourceName(rootResourceName);
    req.setCertHash(computeHash(caCert->hashCode()));
    req.setDirName(dirName);
    req.setCertificate(caCert);
    req.writeExternal(sock.getOutputStream());
   
    CacheResponse resp;

    resp.readExternal(sock.getInputStream());
    return resp.getStatus()[0] == 1;
  } catch(Exception& ex) {
    return false;
  }
}

CacheCertificate
SECI::retrieveCacheCertificate(const string& rootResourceName, 
                               const string& key,
                               const string& dirName) {
  try {
    Socket sock = conns[Akenti::getThreadID()];
    CacheRequest req("LOOKUP");
   
    req.setRootResourceName(rootResourceName);
    req.setCertHash(computeHash(key));
    req.setDirName(dirName);
    req.writeExternal(sock.getOutputStream());
   
    CacheResponse resp;

    resp.readExternal(sock.getInputStream());
    
    string status = resp.getStatus();
    
    if (status[0] == '1') {
      return resp.getCertificate();
    }
  } catch(Exception& ex) { 
    // LOG THIS
  } catch(Error& err) { 
    // XXX
    // LOG THIS
  }

  // XXX
  return CacheCertificate();
}

bool 
SECI::flushCacheCertificates(const string& rootResourceName,
                             const string& key,
                             const string& dirName) {
  try {
    Socket sock = conns[Akenti::getThreadID()];
    CacheRequest req("FLUSH");
  
    req.setRootResourceName(rootResourceName);

    if (key != "ALL") {
      req.setCertHash(computeHash(key));
    }

    req.setDirName(dirName);

    CacheResponse resp;

    resp.readExternal(sock.getInputStream());

    string status = resp.getStatus();

    if (status[0] == '1') {
      return true;
    }
  } catch(Exception& ex) {
  }

  return false;
}

bool 
SECI::closeConnection() {
  int threadID = Akenti::getThreadID();

  if (conns.find(Akenti::getThreadID()) == conns.end()) {
    throw Error("ServerExternalCacheImpl:closeConnection: no connection");
  }

  try {
    conns[threadID].close();
    conns.erase(threadID);
    return true;
  } catch(Exception& ex) {
    return false;
  }
}

