#ifndef AKENTI_REQUEST_HPP
#define AKENTI_REQUEST_HPP

#include "ref_ptr.hpp"
#include "Xerces-Decls.hpp"
#include <string>

using std::string;

namespace Akenti {

/**
  * Base class for <i>AkentiRequest<i>s.
  */
class AkentiRequest {

public:

  /**
    * Constructor.
    */
  AkentiRequest();

  /**
    * Destructor.
    */
  ~AkentiRequest();

  /**
    * Sets the id of this <i>AkentiRequest<i>.
    */
  void setID(int id);

  /**
    * Returns the id of this <i>AkentiRequest<i>.
    */
  int getID() const;
  
  /**
    * Returns an AML string representation.
    */
  string toString() const;


  /**
    * Returns <i>true if this <i>AkentiRequest</i> is of type T.
    */
  template<class T> bool instanceOf() const {
     static T t;

     return t.getTag() == getTag();
  }

protected:
  class AkentiRequestImpl {
  public:
    virtual ~AkentiRequestImpl();
    const string& getTag() const;
    void setID(int id);
    int getID() const;
    void toXML(DOMElement* request) const;
    void fromXML(const DOMElement* request);
  protected:
    AkentiRequestImpl(const string& tag);

    virtual void doToXML(DOMElement* request) const {
    }

    virtual void doFromXML(const DOMElement* request) {
    }

    int id;
    string tag;
  };

  AkentiRequest(AkentiRequestImpl* impl);
  ref_ptr<AkentiRequestImpl> impl;

private:
  friend class AkentiRequests;
  friend class AkentiRequestResponseImplFactory;
  void toXML(DOMElement* request) const;
  void fromXML(const DOMElement* request);
  const string& getTag() const;
  typedef AkentiRequestImpl* ImplType;
};

}

#endif

