#ifndef AKENTI_RESPONSE_IMPL_FACTORY_HPP
#define AKENTI_RESPONSE_IMPL_FACTORY_HPP

#include "CheckAccessRequest.hpp"
#include "CheckPolicyRequest.hpp"
#include "ResourceBrowseRequest.hpp"
#include "CreateResourceRequest.hpp"
#include "UploadPolicyCertificateRequest.hpp"
#include "DeletePolicyCertificateRequest.hpp"
#include "GetResourceInfoRequest.hpp"

#include "CheckAccessResponse.hpp"
#include "CheckPolicyResponse.hpp"
#include "ResourceBrowseResponse.hpp"
#include "UploadPolicyCertificateResponse.hpp"
#include "CreateResourceResponse.hpp"
#include "DeletePolicyCertificateResponse.hpp"
#include "GetResourceInfoResponse.hpp"
#include "Exception.hpp"

namespace Akenti {

/*
 * Internal class which creates specific kinds of AkentiRequestImpl and 
 * AkentiResponseImpl. 
 */

class AkentiRequestResponseImplFactory {

private:

  /**
    */
  static AkentiRequest::AkentiRequestImpl* getRequestImpl(const string& name) {
    if (name == "CheckAccessRequest") {
      return CheckAccessRequest::getRequestImpl();
    } else if (name == "CheckPolicyRequest") {
      return CheckPolicyRequest::getRequestImpl();
    }  else if (name == "ResourceBrowseRequest") {
      return ResourceBrowseRequest::getRequestImpl();
    } else if (name == "CreateResourceRequest") {
      return CreateResourceRequest::getRequestImpl();
    } else if (name == "UploadPolicyCertificateRequest") {
      return UploadPolicyCertificateRequest::getRequestImpl();
    } else if (name == "DeletePolicyCertificateRequest") {
      return DeletePolicyCertificateRequest::getRequestImpl();
    } else if (name == "GetResourceInfoRequest") {
      return GetResourceInfoRequest::getRequestImpl();
    } 
   
    throw Exception("AkentiRequestResponseImplFactory:getRequestImpl:no impl for " + name);
  }

  /**
    */
  static AkentiResponse::AkentiResponseImpl* getResponseImpl(const string& name) {
    if (name == "CheckAccessResponse") {
      return CheckAccessResponse::getResponseImpl();
    } else if (name == "CheckPolicyResponse") {
      return CheckPolicyResponse::getResponseImpl();
    } else if (name == "ResourceBrowseResponse") {
      return ResourceBrowseResponse::getResponseImpl();
    } else if (name == "CreateResourceResponse") {
      return CreateResourceResponse::getResponseImpl();
    } else if (name == "UploadPolicyCertificateResponse") {
      return UploadPolicyCertificateResponse::getResponseImpl();
    } else if (name == "DeletePolicyCertificateResponse") {
      return DeletePolicyCertificateResponse::getResponseImpl();
    } else if (name == "GetResourceInfoResponse") {
      return GetResourceInfoResponse::getResponseImpl();
    }

    throw Exception("AkentiRequestResponseImplFactory::getResponseImpl:no impl for " + name);
  }

  friend class AkentiResponses;
  friend class AkentiRequests;

  AkentiRequestResponseImplFactory();
  
};

}

#endif

