#include <sstream>
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "AkentiResponse.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using std::ostringstream;
using std::istringstream;

namespace Akenti {

AkentiResponse::AkentiResponseImpl::AkentiResponseImpl(const string& tag) 
  : tag(tag) , id(-1) {
  if (!tag.length()) {
    throw Error("AkentiResponseImpl::AkentiResponseImpl:illegal arg");
  }
}

AkentiResponse::AkentiResponseImpl::~AkentiResponseImpl() {
}

const string&
AkentiResponse::AkentiResponseImpl::getTag() const {
  return tag;
}

void 
AkentiResponse::AkentiResponseImpl::setID(int id) {
  this->id = id;
}

int
AkentiResponse::AkentiResponseImpl::getID() const {
  return id;
}

void
AkentiResponse::AkentiResponseImpl::setAkentiMessage(const AkentiMessage& mesg) {
  this->mesg = mesg;
}

const AkentiMessage&
AkentiResponse::AkentiResponseImpl::getAkentiMessage() const {
  return mesg;
}

void 
AkentiResponse::AkentiResponseImpl::toXML(DOMElement* response) const {
  DOMDocument *doc = response->getOwnerDocument();
  DOMElement *element = doc->createElement(XMLUtil::convert("ID"));

  char buffer[30];
  sprintf(buffer, "%d", id);
  
  element->appendChild(doc->createTextNode(XMLUtil::convert(buffer)));
  response->appendChild(element);
  element = doc->createElement(XMLUtil::convert("AkentiMessage"));
  mesg.toXML(element);
  response->appendChild(element);
  doToXML(response);
}

void 
AkentiResponse::AkentiResponseImpl::fromXML(const DOMElement* response) {
  DOMNode *child = response->getFirstChild();
  DOMNode *temp  = child->getFirstChild();

  id = atoi(XMLUtil::convert(temp->getNodeValue()).c_str());
  child = child->getNextSibling();
  mesg.fromXML((DOMElement*) child);
  doFromXML(response);
}

AkentiResponse::AkentiResponse() {
}

AkentiResponse::~AkentiResponse() {
}

AkentiResponse::AkentiResponse(AkentiResponseImpl* impl) : impl(impl) {
  if (!impl) {
    throw Error("AkentiResponse:AkentiResponse:illegal arg");
  }
}

const string&
AkentiResponse::getTag() const {
  return impl->getTag();
}

void 
AkentiResponse::setID(int id) {
  impl->setID(id);
}

int
AkentiResponse::getID() const {
  return impl->getID();
}

void 
AkentiResponse::setAkentiMessage(const AkentiMessage& mesg) {
  return impl->setAkentiMessage(mesg);
}

const AkentiMessage&
AkentiResponse::getAkentiMessage() const {
  return impl->getAkentiMessage();
}

string
AkentiResponse::toString() const {
  DOMImplementation *domImpl = DOMImplementationRegistry::getDOMImplementation(XMLUtil::convert("Core"));
  DOMDocument* doc = domImpl->createDocument(0, XMLUtil::convert("AkentiResponse"), 0);
  DOMElement* root = doc->getDocumentElement();
  ostringstream os;

  impl->toXML(root);
  XMLUtil::write(os, doc);
  delete(doc);
  return os.str();
}

void 
AkentiResponse::toXML(DOMElement* response) const {
  impl->toXML(response);
}

void 
AkentiResponse::fromXML(const DOMElement* response) {
  impl->fromXML(response);
}

}
