#include <sstream>
#include <iostream>
#include "Exception.hpp"
#include "CertificateInfo.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "OpaqueCertificate.hpp"
#include "AkentiMessage.hpp"

using namespace std;

namespace Akenti {

typedef CertificateInfo CI;

CI::CertificateInfo() : used(false), statusMessages(1), location("file:/no_location") {
}

void 
CI::setUsed(bool used){
  this->used = used;
}

bool 
CI::getUsed() const {
  return used;
}

bool
CI::hasCertificate() const {
  return cert != NULL;
}

void
CI::setCertificate(const AkentiCertificate& cert) {
  this->cert = cert;
}

const AkentiCertificate&
CI::getCertificate() const {
  if (!cert) {
    throw Error("CertificateInfo:getCertificate: this object has no certificate");
  }

  return cert;
}

void
CI::setLocation(const string& location) {
  this->location = location;
}

const string&
CI::getLocation() const {
  return this->location;
}

const vector<AkentiMessage>&
CI::getCertificateMessages() const {
  return statusMessages;
}

void
CI::setCertificateMessages(const vector<AkentiMessage>& statusMessages) {
  this->statusMessages = statusMessages;
}

void 
CI::toXML(DOMElement* certificateInfoNode) const {
  certificateInfoNode->setAttribute(XMLUtil::convert("Used"),
                                    used? XMLUtil::convert("true"):
                                          XMLUtil::convert("false"));

  DOMDocument* doc = certificateInfoNode->getOwnerDocument();

  if (cert) {
    DOMElement* certNode = doc->createElement(XMLUtil::convert("OpaqueCertificate"));

    OpaqueCertificate::toXML(cert, certNode);
    certificateInfoNode->appendChild(certNode);
  }

  DOMElement* locationNode = doc->createElement(XMLUtil::convert("Location"));

  locationNode->appendChild(doc->createTextNode(XMLUtil::convert(location.c_str())));
  certificateInfoNode->appendChild(locationNode);

  for (int i = 0; i < statusMessages.size(); i++) {
    DOMElement* messageNode = doc->createElement(XMLUtil::convert("CertificateMessage"));

    statusMessages[i].toXML(messageNode);
    certificateInfoNode->appendChild(messageNode);
  }
}

void 
CI::fromXML(const DOMElement* certificateInfoNode) {
  const XMLCh* temp = certificateInfoNode->getAttribute(XMLUtil::convert("Used"));
  string rep = temp? XMLUtil::convert(temp) : "false";
  used = rep == "true";

  DOMNodeList* list = certificateInfoNode->getElementsByTagName(XMLUtil::convert("OpaqueCertificate"));

  if (list->getLength() == 1) {
    cert = OpaqueCertificate::fromXML((DOMElement*) list->item(0));
  }

  list = certificateInfoNode->getElementsByTagName(XMLUtil::convert("Location"));
  location = XMLUtil::convert(list->item(0)->getFirstChild()->getNodeValue());

  list = certificateInfoNode->getElementsByTagName(XMLUtil::convert("CertificateMessage"));

  statusMessages.clear();

  for (int i = 0; i < list->getLength(); i++) {
    AkentiMessage mesg;

    mesg.fromXML((DOMElement*) list->item(i));
    statusMessages.push_back(mesg);
  }
}

}
