#ifndef CERTIFICATE_INFO_HPP
#define CERTIFICATE_INFO_HPP

#include <vector>
#include "Xerces-Decls.hpp"
#include "AkentiMessage.hpp"
#include "AkentiCertificate.hpp"

using std::vector;

namespace Akenti {

/**
  * CertificateInfo holds (possibly) a certificate and info about it.
  *
  * @see CheckPolicyResponse
  */
class CertificateInfo {

public:

  /** 
    * Default constructor.
    */
  CertificateInfo();

  /** 
    *  Sets if certificate is used.
    */
  void setUsed(bool used);

  /** 
    *  Returns true if certificate is used.
    */
  bool getUsed() const;

  /** 
    *  Returns true if there is a certificate present;
    */
  bool hasCertificate() const;

  /** 
    *  Add a certificate to this Certificate Info
    */
  void setCertificate(const AkentiCertificate& cert);

  /** 
    * Retrieves a certificate from this Certificate Info.
    */
  const AkentiCertificate& getCertificate() const;

  /** 
    * Sets a location.
    */
  void setLocation(const string& location);

  /** 
    * Returns a location
    */
  const string& getLocation() const;

  /** 
    * Returns the vector of certificate status messages.
    */
  const vector<AkentiMessage>& getCertificateMessages() const;

  /** 
    * Sets the vector of certificate status messages.
    */
  void setCertificateMessages(const vector<AkentiMessage>& certMessages);

  /** 
    * Populates the DOMElement from this <i> PolicyResponse</i>.
    */
   void toXML(DOMElement* certificateInfoNode) const;

  /** 
    * Initializes this <i>PolicyResponse<i> from the DOMElement.
    */
   void fromXML(const DOMElement* certificateInfoNode);

private:

  bool used;
  AkentiCertificate cert;
  string location;
  vector<AkentiMessage> statusMessages;
};

}

#endif
