#include "CheckAccessRequest.hpp"
#include "Certificate.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "AkentiPrincipal.hpp"
#include "Certificate.hpp"
#include "Error.hpp"
#include "Exception.hpp"

namespace Akenti {

class CheckAccessRequest::CheckAccessRequestImpl : public AkentiRequestImpl {

public:

  CheckAccessRequestImpl() : AkentiRequestImpl("CheckAccessRequest"), 
                             ap(new AkentiPrincipal), 
                             certs(NULL) {
  }

  ~CheckAccessRequestImpl() {
    destroy();
  }

  void destroy() {
    delete(ap);
    delete(certs);
    ap = NULL;
    certs = NULL;
  }

  void setResource(const string& resource) {
    this->resource = resource;
  }

  const string& getResource() const {
    return resource;
  }

  void setAkentiPrincipal(const AkentiPrincipal& ap) {
    destroy();
    this->ap = new AkentiPrincipal(ap);
  }

  const AkentiPrincipal& getAkentiPrincipal() const  {
    if (!ap) {
      throw Error("CheckAccessRequestImpl:getAkentiPrincipal:Illegal operation");
    }

    return *ap;
  }

  bool containsX509Chain() const {
    return certs != NULL;
  }

  void setX509Chain(const vector<Certificate> certs) {
    destroy();
    this->certs = new vector<Certificate>(certs); 
  }

  const vector<Certificate>& getX509Chain() const {
    if (!certs) {
      throw Error("CheckAccessRequestImpl:getX509Chain:illegal operation");
    }
  
    return *certs;
  }

  void doToXML(DOMElement* request) const {
    DOMDocument* doc = request->getOwnerDocument();
    DOMElement* child = doc->createElement(XMLUtil::convert("Resource"));

    child->appendChild(doc->createTextNode(XMLUtil::convert(resource.c_str())));
    request->appendChild(child);
  
    if (certs) {
      child = doc->createElement(XMLUtil::convert("X509CertificateChain"));
      throw Error("CheckAccessRequestImpl::doToXML:TODO");
    } else {
      child = doc->createElement(XMLUtil::convert("Principal"));
      ap->toXML(child);
      request->appendChild(child);
    }
  }
 
  void doFromXML(const DOMElement* request) {
    destroy();

    DOMNode *child = request->getFirstChild()->getNextSibling();
    DOMNode *temp = child->getFirstChild();

    resource = XMLUtil::convert(temp->getNodeValue());
    child = child->getNextSibling();

    if (XMLString::compareString(child->getNodeName(), XMLUtil::convert("Principal")) == 0) {
      ap = new AkentiPrincipal();
      ap->fromXML((DOMElement*) child);
    } else {
      throw Error("CheckAccessRequestImpl::doFromXML:TODO");
    }
  }

private:
  string resource;     
  AkentiPrincipal *ap; 
  vector<Certificate>* certs; 
};

CheckAccessRequest::CheckAccessRequest() 
  : AkentiRequest(getRequestImpl()) {
}

void 
CheckAccessRequest::setResource(const string& resource) {
  CheckAccessRequestImpl* temp = (CheckAccessRequestImpl *) impl.get();

  temp->setResource(resource);
}

const string&
CheckAccessRequest::getResource() const {
  CheckAccessRequestImpl* temp = (CheckAccessRequestImpl *) impl.get();

  return temp->getResource();
}

bool 
CheckAccessRequest::containsX509Chain() const {
  CheckAccessRequestImpl* temp = (CheckAccessRequestImpl *) impl.get();

  return temp->containsX509Chain();
}

void 
CheckAccessRequest::setAkentiPrincipal(const AkentiPrincipal& ap) {
  CheckAccessRequestImpl* temp = (CheckAccessRequestImpl *) impl.get();

  temp->setAkentiPrincipal(ap);
}

const AkentiPrincipal& 
CheckAccessRequest::getAkentiPrincipal() const  {
  CheckAccessRequestImpl* temp = (CheckAccessRequestImpl *) impl.get();

  return temp->getAkentiPrincipal();
}


void 
CheckAccessRequest::setX509Chain(const vector<Certificate>& certs) {
  CheckAccessRequestImpl* temp = (CheckAccessRequestImpl *) impl.get();

  temp->setX509Chain(certs);
}

const vector<Certificate>& 
CheckAccessRequest::getX509Chain() const {
  CheckAccessRequestImpl* temp = (CheckAccessRequestImpl *) impl.get();

  return temp->getX509Chain();
}

AkentiRequest::AkentiRequestImpl*
CheckAccessRequest::getRequestImpl() {
  return new CheckAccessRequestImpl;
}

}
