#include <sstream>
#include "CheckAccessResponse.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "Error.hpp"
#include "Exception.hpp"

using namespace std;

namespace Akenti {

class CheckAccessResponse::CheckAccessResponseImpl : public AkentiResponseImpl {

public:

  CheckAccessResponseImpl() : AkentiResponseImpl("CheckAccessResponse") {
  }

  ~CheckAccessResponseImpl() {
  }

  void setCapabilityCertificate(const CapabilityCertificate& cert) {
    if (!cert) {
      throw Error("CheckAccessResponseImpl:setCapabilityCertificate:illegal arg");
    }

    this->cert = cert;
  }

  const CapabilityCertificate& getCapabilityCertificate() const {
    return cert;
  }

  void doToXML(DOMElement* response) const {
    if (cert) {
      DOMDocument* doc = response->getOwnerDocument();
      DOMElement* child = doc->createElement(XMLUtil::convert("AkentiOpaqueCapability"));
      ostringstream os;

      AkentiCertificate::writeToStream("PEM", os, cert);

      // TODO More efficiently
      // REMOVE PEM TAGS
      string pemCert = os.str();
      int idx1 = pemCert.find('\n'); 
      int idx2 = pemCert.find("\n-", idx1);
    
      pemCert = pemCert.substr(idx1, idx2 - idx1);
      child->appendChild(doc->createTextNode(XMLUtil::convert(pemCert.c_str())));
      response->appendChild(child);
   }
 }
 
  void doFromXML(const DOMElement* response) {
    DOMNode* child = response->getFirstChild()->getNextSibling()->getNextSibling();
  
    if (child != NULL) {
      child = child->getFirstChild();

      string pemCert = "-----BEGIN AKENTI CAPABILITY CERTIFICATE-----";
      string rep = XMLUtil::convert(child->getNodeValue());

      if (rep[0] != '\n') {
        pemCert += '\n';
      }
    
      pemCert += rep;

      // ADD PEM TAGS

      if (rep[rep.length() - 1] != '\n') {
        pemCert += '\n';
      }
     
      pemCert += "-----END AKENTI CAPABILITY CERTIFICATE-----";

      istringstream is(pemCert);

      AkentiCertificate akCert = AkentiCertificate::initFromStream("PEM", is);

      if (!akCert.instanceOf<CapabilityCertificate>()) {
        throw Exception("CheckAccessResponseImpl:doFromXML:expected cap cert");
      }

      cert = (CapabilityCertificate&) akCert;
    }
  }

private:

  CapabilityCertificate cert;

};

CheckAccessResponse::CheckAccessResponse() 
  : AkentiResponse(getResponseImpl()) {
}

void 
CheckAccessResponse::setCapabilityCertificate(const CapabilityCertificate& cert) {
  CheckAccessResponseImpl* temp = (CheckAccessResponseImpl*) impl.get();
  
  temp->setCapabilityCertificate(cert);
}

const CapabilityCertificate&
CheckAccessResponse::getCapabilityCertificate() const {
  CheckAccessResponseImpl* temp = (CheckAccessResponseImpl*) impl.get();

  return temp->getCapabilityCertificate();
}

AkentiResponse::AkentiResponseImpl*
CheckAccessResponse::getResponseImpl() {
  return new CheckAccessResponseImpl;
}

}
