#include "CheckPolicyRequest.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "Error.hpp"
#include "Exception.hpp"

namespace Akenti {

class CheckPolicyRequest::CheckPolicyRequestImpl : public AkentiRequestImpl  {

public:

  CheckPolicyRequestImpl() : AkentiRequestImpl("CheckPolicyRequest") {
  }

  ~CheckPolicyRequestImpl() {
  }

  void setResource(const string& resource) {
    this->resource = resource;
  }

  const string& getResource() const {
    return resource;
  }

  void setStopOnFatalError(bool stopOnFatalError) {
    this->stopOnFatalError = stopOnFatalError;
  }

  bool getStopOnFatalError() {
    return stopOnFatalError;
  }

  void doToXML(DOMElement* request) const {
    request->setAttribute(XMLUtil::convert("StopOnFatalError"), 
                          XMLUtil::convert(stopOnFatalError? "true" : "false")); 
 
    DOMDocument *doc = request->getOwnerDocument();
    DOMElement *child = doc->createElement(XMLUtil::convert("Resource"));
 
    child->appendChild(doc->createTextNode(XMLUtil::convert(resource.c_str())));
    request->appendChild(child);
  }
 
  void doFromXML(const DOMElement* request) {
    DOMElement* child = (DOMElement*) request->getFirstChild()->getNextSibling();
    const XMLCh* temp = child->getAttribute(XMLUtil::convert("StopOnFatalError"));
    string rep = temp? XMLUtil::convert(temp) : "false";
    stopOnFatalError = rep == "true";
    stopOnFatalError = false;
 
    DOMNode *resourceNode = child->getFirstChild();
 
    resource = XMLUtil::convert(resourceNode->getNodeValue());
  }

private:

  string resource;
  bool stopOnFatalError;
};

CheckPolicyRequest::CheckPolicyRequest() : AkentiRequest(getRequestImpl()) {
}

void 
CheckPolicyRequest::setResource(const string& resource) {
  CheckPolicyRequestImpl* temp = (CheckPolicyRequestImpl*) impl.get();

  temp->setResource(resource);
}

const string&
CheckPolicyRequest::getResource() const {
  CheckPolicyRequestImpl* temp = (CheckPolicyRequestImpl*) impl.get();

  return temp->getResource();
}

AkentiRequest::AkentiRequestImpl*
CheckPolicyRequest::getRequestImpl() {
  return new CheckPolicyRequestImpl;
}

void 
CheckPolicyRequest::setStopOnFatalError(bool stopOnFatalError) {
  CheckPolicyRequestImpl* temp = (CheckPolicyRequestImpl*) impl.get();
 
  temp->setStopOnFatalError(stopOnFatalError);
}
 
bool 
CheckPolicyRequest::getStopOnFatalError() const {
  CheckPolicyRequestImpl* temp = (CheckPolicyRequestImpl*) impl.get();
 
  return temp->getStopOnFatalError();
}

}
