#include "CheckPolicyResponse.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "Error.hpp"
#include "Exception.hpp"

using std::vector;

namespace Akenti {

class CheckPolicyResponse::CheckPolicyResponseImpl : public AkentiResponseImpl {
 
public:

  CheckPolicyResponseImpl() : AkentiResponseImpl("CheckPolicyResponse") {
  }

  ~CheckPolicyResponseImpl() {
  }

  const vector<PolicyInfo>& getPolicyInfos() const {
    return policyInfos;
  }

  void setPolicyInfos(const vector<PolicyInfo>& policyInfos) {
    this->policyInfos = policyInfos;
  }

  void doToXML(DOMElement* response) const {
    DOMDocument *doc = response->getOwnerDocument();

    for (int i = 0; i < policyInfos.size(); i++) {
      DOMElement *child = doc->createElement(XMLUtil::convert("PolicyInfo"));

      policyInfos[i].toXML(child);
      response->appendChild(child);
    }
  }
 
  void doFromXML(const DOMElement* response) {
    policyInfos.clear();

    DOMNodeList* list = response->getElementsByTagName(XMLUtil::convert("PolicyInfo"));

    for (int i = 0; i < list->getLength(); i++) {
      PolicyInfo pi;
      DOMNode* child = list->item(i);

      pi.fromXML((DOMElement*) child);
      policyInfos.push_back(pi);
    }
  }

private:
  
  vector<PolicyInfo> policyInfos;

};

typedef CheckPolicyResponse CPR;

CPR::CheckPolicyResponse() : AkentiResponse(getResponseImpl()) {
}

const vector<PolicyInfo>& CPR::getPolicyInfos() const {
  CheckPolicyResponseImpl* temp = (CheckPolicyResponseImpl *) impl.get();

  return temp->getPolicyInfos();
} 

void CPR::setPolicyInfos(const vector<PolicyInfo>& policyInfos) {
  CheckPolicyResponseImpl* temp = (CheckPolicyResponseImpl *) impl.get();

  temp->setPolicyInfos(policyInfos);
} 

AkentiResponse::AkentiResponseImpl* CPR::getResponseImpl() {
  return new CheckPolicyResponseImpl;
}

}
