#include "CreateResourceRequest.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"

namespace Akenti {

class CreateResourceRequest::CreateResourceRequestImpl : public AkentiRequestImpl {

public:

  CreateResourceRequestImpl() : AkentiRequestImpl("CreateResourceRequest") {
  }

  ~CreateResourceRequestImpl() {
  }

  void setResource(const string& resource) {
    this->resource = resource;
  }

  const string& getResource() const {
    return resource;
  }

  void doToXML(DOMElement* request) const {
    DOMDocument *doc = request->getOwnerDocument();
    DOMElement *child = doc->createElement(XMLUtil::convert("Resource"));

    child->appendChild(doc->createTextNode(XMLUtil::convert(resource.c_str())));
    request->appendChild(child);
  }
 
  void doFromXML(const DOMElement* request) {
    DOMNode *child = request->getFirstChild()->getNextSibling();
    DOMNode *temp = child->getFirstChild();

    resource = XMLUtil::convert(temp->getNodeValue());
  }

private:

  string resource;
};

typedef CreateResourceRequest CRR;

CRR::CreateResourceRequest() 
  : AkentiRequest(getRequestImpl()) {
}

void 
CRR::setResource(const string& resource) {
  CreateResourceRequestImpl* temp = (CreateResourceRequestImpl *) impl.get();

  temp->setResource(resource);
}

const string&
CRR::getResource() const {
  CreateResourceRequestImpl* temp = (CreateResourceRequestImpl *) impl.get();

  return temp->getResource();
}

AkentiRequest::AkentiRequestImpl*
CRR::getRequestImpl() {
  return new CreateResourceRequestImpl;
}

}
