#include "DeletePolicyCertificateRequest.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "Error.hpp"

namespace Akenti {

class DeletePolicyCertificateRequest::DeletePolicyCertificateRequestImpl 
  : public AkentiRequestImpl {

public:

  DeletePolicyCertificateRequestImpl() 
    : AkentiRequestImpl("DeletePolicyCertificateRequest") {
  }

  ~DeletePolicyCertificateRequestImpl() {
  }

  void setResource(const string& resource) {
    if (resource.length() == 0) {
      throw Error("DeletePolicyCertificateRequestImpl:setResource:length = 0");
    }

    this->resource = resource;
  }

  const string& getResource() const {
    return resource;
  }

  void DeletePolicyCertificateRequestImpl::doToXML(DOMElement* request) const {
    DOMDocument* doc = request->getOwnerDocument();
    DOMElement* resourceNode = doc->createElement(XMLUtil::convert("Resource"));

    resourceNode->appendChild(doc->createTextNode(XMLUtil::convert(resource.c_str())));
    request->appendChild(resourceNode);
  }
 
  void doFromXML(const DOMElement* request) {
    DOMNode* resourceNode = request->getFirstChild()->getNextSibling();

    resource = XMLUtil::convert(resourceNode->getFirstChild()->getNodeValue());
  }

private:
 
string resource;

};

typedef DeletePolicyCertificateRequest DPCR;

DPCR::DeletePolicyCertificateRequest() : AkentiRequest(getRequestImpl()) {
}

void 
DPCR::setResource(const string& resource) {
  DeletePolicyCertificateRequestImpl* temp = (DeletePolicyCertificateRequestImpl*) impl.get();

  temp->setResource(resource);
}

const string&
DPCR::getResource() const {
  DeletePolicyCertificateRequestImpl* temp = (DeletePolicyCertificateRequestImpl*) impl.get();

  return temp->getResource();
}

AkentiRequest::AkentiRequestImpl*
DPCR::getRequestImpl() {
  return new DeletePolicyCertificateRequestImpl;
}

}
