#include "ResourceBrowseRequest.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"

namespace Akenti {

class ResourceBrowseRequest::ResourceBrowseRequestImpl : public AkentiRequestImpl {

public:

  ResourceBrowseRequestImpl() : AkentiRequestImpl("ResourceBrowseRequest") {
  }

  ~ResourceBrowseRequestImpl() {
  }

  void setResource(const string& resource) {
    this->resource = resource;
  }

  const string& getResource() const {
    return resource;
  }

  void doToXML(DOMElement* request) const {
    DOMDocument *doc = request->getOwnerDocument();
    DOMElement *child = doc->createElement(XMLUtil::convert("Resource"));

    child->appendChild(doc->createTextNode(XMLUtil::convert(resource.c_str())));
    request->appendChild(child);
  }
 
  void doFromXML(const DOMElement* request) {
    DOMNode *child = request->getFirstChild()->getNextSibling();
    DOMNode *temp = child->getFirstChild();

    resource = temp? XMLUtil::convert(temp->getNodeValue()) : "";
  }

private:
  string resource;
};

typedef ResourceBrowseRequest RBR;

RBR::ResourceBrowseRequest() : AkentiRequest(getRequestImpl()) {
}

void 
RBR::setResource(const string& resource) {
  ResourceBrowseRequestImpl* temp = (ResourceBrowseRequestImpl *) impl.get();

  temp->setResource(resource);
}

const string&
RBR::getResource() const {
  ResourceBrowseRequestImpl* temp = (ResourceBrowseRequestImpl *) impl.get();

  return temp->getResource();
}

AkentiRequest::AkentiRequestImpl*
RBR::getRequestImpl() {
  return new ResourceBrowseRequestImpl;
}

}
