#include "ResourceBrowseResponse.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "Error.hpp"
#include "Exception.hpp"

namespace Akenti {

class ResourceBrowseResponse::ResourceBrowseResponseImpl 
  : public AkentiResponseImpl {

public: 

  ResourceBrowseResponseImpl() 
    : AkentiResponseImpl("ResourceBrowseResponse") {
  }

  ~ResourceBrowseResponseImpl() {
  }

  void
  setResources(const vector<string>& resources) {
    this->resources = resources;
  }

  const vector<string>&
  getResources() const{
    return resources;
  }

  void ResourceBrowseResponseImpl::doToXML(DOMElement* response) const{
    DOMDocument *doc = response->getOwnerDocument();

    for (int i = 0; i < resources.size(); i++) {
      DOMElement *child = doc->createElement(XMLUtil::convert("Resource"));

      child->appendChild(doc->createTextNode(XMLUtil::convert(resources[i].c_str())));
      response->appendChild(child);
    }
  }

  void ResourceBrowseResponseImpl::doFromXML(const DOMElement* response) {
    DOMNode *child = response->getFirstChild()->getNextSibling()->getNextSibling();

    for (; child; child = child->getNextSibling()) {
      resources.push_back(XMLUtil::convert(child->getFirstChild()->getNodeValue()));
    }
  }

private:

  vector<string> resources;
};

ResourceBrowseResponse::ResourceBrowseResponse() 
  : AkentiResponse(getResponseImpl()) {
}

void 
ResourceBrowseResponse::setResources(const vector<string>& resources) {
  ResourceBrowseResponseImpl* temp = (ResourceBrowseResponseImpl *) impl.get();
  
  temp->setResources(resources);
}

const vector<string>&
ResourceBrowseResponse::getResources() const {
  ResourceBrowseResponseImpl* temp = (ResourceBrowseResponseImpl *) impl.get();

  return temp->getResources();
}

AkentiResponse::AkentiResponseImpl*
ResourceBrowseResponse::getResponseImpl() {
  return new ResourceBrowseResponseImpl;
}

}
