#include "UploadPolicyCertificateRequest.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "OpaqueCertificate.hpp"
#include "Exception.hpp"

namespace Akenti {

class UploadPolicyCertificateRequest::UploadPolicyCertificateRequestImpl 
  : public AkentiRequestImpl {

public:

  UploadPolicyCertificateRequestImpl() 
    : AkentiRequestImpl("UploadPolicyCertificateRequest") {
  }

  ~UploadPolicyCertificateRequestImpl() {
  }

  void setCertificate(const PolicyCertificate& polCert){
    this->polCert = polCert;
  }

  const PolicyCertificate& getCertificate() const {
    return polCert;
  }

  void doToXML(DOMElement* request) const {
    DOMDocument* doc = request->getOwnerDocument();
    DOMElement* certNode = doc->createElement(XMLUtil::convert("OpaqueCertificate"));

    OpaqueCertificate::toXML(polCert, certNode);
    request->appendChild(certNode);
  }
 
  void doFromXML(const DOMElement* request) {
    DOMNode* certNode = request->getFirstChild()->getNextSibling();
    AkentiCertificate cert = OpaqueCertificate::fromXML((DOMElement*) certNode);

    if (!cert.instanceOf<PolicyCertificate>()) {
      throw Exception("UploadPolicyCertificateRequestImpl:doFromXML:invalid cert (wrong type)");
    }

    polCert =(PolicyCertificate&) cert;
  }

private:

  PolicyCertificate polCert;
};

typedef UploadPolicyCertificateRequest UPCR;

UPCR::UploadPolicyCertificateRequest() 
  : AkentiRequest(getRequestImpl()) {
}

void 
UPCR::setCertificate(const PolicyCertificate& polCert) {
  UploadPolicyCertificateRequestImpl* temp = (UploadPolicyCertificateRequestImpl*) impl.get();

  temp->setCertificate(polCert);
}

const PolicyCertificate&
UPCR::getCertificate() const {
  UploadPolicyCertificateRequestImpl* temp = (UploadPolicyCertificateRequestImpl*) impl.get();

  return temp->getCertificate();
}

AkentiRequest::AkentiRequestImpl*
UPCR::getRequestImpl() {
  return new UploadPolicyCertificateRequestImpl;
}

}

