#include "HttpRequest.hpp"
#include "HttpConstants.hpp"
#include "StringTokenizer.hpp"
#include "StringUtil.hpp"
#include "IOException.hpp"
#include <sstream>
 
using namespace std;

namespace Akenti {

HttpRequest::HttpRequest() : method("GET"), uri("/") {
}

HttpRequest::~HttpRequest() { 
}

const string&
HttpRequest::getURI() const {
  return uri;
}

void
HttpRequest::setURI(const string& uri) {
  if (!uri.length()) {
    throw Error("HttpRequest:setURI:illegal arg");
  }
 
  this->uri = uri;
}

void
HttpRequest::setMethod(const string& method) {
  if (!method.length()) {
    throw Error("HttpRequest:setMethod:illegal arg");
  }

  this->method = method;
}

const string&
HttpRequest::getMethod() const {
  return method;
}

InputStream 
HttpRequest::getInputStream() const {
  return HttpBase::getInputStream();
}


OutputStream 
HttpRequest::getOutputStream() {
  return HttpBase::getOutputStream();
}

void
HttpRequest::fromStream(InputStream& is) {
  string line;

  readLine(line, is);

  StringTokenizer tokenizer(line, " ");

  switch(tokenizer.countTokens()) {
    case 2:
      method = tokenizer.nextToken();
      protocol = tokenizer.nextToken();
      break;
    case 3:
      method = tokenizer.nextToken();
      uri = tokenizer.nextToken();
      protocol = tokenizer.nextToken();
      break;
    default:
      throw Exception("HttpRequest:fromStream:failed reading header");
  }

  do {
    readLine(line, is);

    if (line.length() == 0) {
      break;
    }

    StringTokenizer tokenizer(line, ": ;");

    if (tokenizer.countTokens() < 2) {
      throw Exception("HttpRequest:fromStream:failed reading header");
    }

    string token = tokenizer.nextToken();
    
    if (StringUtil::equalsIgnoreCase(token, "content-type")) {
      contentType = tokenizer.nextToken();
    } else if (StringUtil::equalsIgnoreCase(token, "content-length")) {
      istringstream is(tokenizer.nextToken());

      is >> contentLength;
    }
  } while (true);

  if (!contentLength) {
    return;
  }

  string body(contentLength, '\0');

  is->read((char*) body.c_str(), contentLength);

  if (!is->good()) {
    throw IOException("HttpRequest:fromStream:failed reading body");
  }

  this->is = new istringstream(body);
}

void
HttpRequest::toStream(OutputStream& os) const {
  os << method << " " << uri << " " << protocol << CRLF;

  if (contentType.length() != 0) {
    os << "Content-type" << ": " << contentType << CRLF;
  }

  int contentLength = (obuff != NULL) ? obuff->str().length() : 0;

  if (!contentLength) {
    os << CRLF;
    return;
  }

  os << "Content-length" << ": " << contentLength << CRLF;
  os << CRLF;
  os << obuff->str();
}

} // end of name space
