#include <fstream>
#include "FileURLHandlerImpl.hpp"
#include "URLConnection.hpp"
#include "URL.hpp"
#include "IOException.hpp"

namespace Akenti {

class FileURLConnectionImpl : public URLConnectionImpl {

public:
 
  FileURLConnectionImpl(const URL& url) : URLConnectionImpl(url) {
  }

  ~FileURLConnectionImpl() {
  }

protected:

  void doOpen() {
  }

  InputStream doGetInputStream() {
    string filename = getURL().getFile();
    InputStream ins = new std::ifstream(filename.c_str(), std::ios::in);
    
    if (!ins) {
      throw Error("FileURLConnectionImpl:doGetInputStream:bad alloc");
    } else if (!ins->good()) {
      throw IOException("FileURLConnectionImpl:doGetInputStream:bad input stream");
    }

    return ins;
  }

  OutputStream doGetOutputStream() {
    throw IOException("FileURLConnectionImpl:doGetOutputStream:does not handle output");
  }

  string doGetContent(InputStream& ins) {
    ins->seekg(0, std::ios::end);
   
    int len = ins->tellg();
    
    string content(len, '\0');
    
    ins->seekg(0, std::ios::beg);

    if (!ins->read((char *) content.data(), len)) {
      throw IOException("FileConnectionImpl:doGetContent:read failed");
    }

    return content;
  } 

  void doClose() {
    ((std::ifstream*) getInputStream().get())->close();
  }

};

typedef Akenti::FileURLHandlerImpl FUHI;

FUHI::FileURLHandlerImpl() : URLHandlerImpl() {
}

FUHI::~FileURLHandlerImpl() {
}

URLConnection 
FUHI::getURLConnection(const URL& url) const {
  return URLConnection(new FileURLConnectionImpl(url));
}

} // end namespace Akenti
