#include "HttpURLHandlerImpl.hpp"
#include "URLConnection.hpp"
#include "URL.hpp"
#include "Socket.hpp"
#include "IOException.hpp"

namespace Akenti {

class HttpURLConnectionImpl : public URLConnectionImpl {

public:
 
  HttpURLConnectionImpl(const URL& url) 
    : URLConnectionImpl(url) , sock(NULL) {
  }

  ~HttpURLConnectionImpl() {
  }

protected:

  void doOpen() {
    const URL& url = getURL();
    int port = url.getPort();

    if (port < 0) {
      port = 80;
    }

    sock = Socket(url.getHost(), port);
    sock.connect();
  }

  InputStream doGetInputStream() {
    return sock.getInputStream();
  }

  OutputStream doGetOutputStream() {
    return sock.getOutputStream();
  }

  string doGetContent(InputStream& ins) {
    string request = "GET";
    const URL& url = getURL();

    request += ' ';
    request += url.getFile();
    request += " HTTP/1.0";
    request += '\r';
    request += '\n';
    request += "Host: ";
    request += url.getHost();
    request += '\r';
    request += '\n';
    request += '\r';
    request += '\n';

    OutputStream outs = getOutputStream();
    
    outs->write(request.c_str(), request.length());

    if (outs->fail()) {
      throw IOException("HttpURLConnectionImpl:doGetContent:write failed");
    }

    string result;
    char buffer[1024];

    do {
      ins->read(buffer, 1024);
      
/*
      NOTE: Ignore it for now unless you are really sure what is happening
   	    behind the scene

      if (ins->fail()) {
        throw IOException("HttpURLConnectionImpl:doGetContent:read failed");
      }
*/

      result += string(buffer, ins->gcount());
    } while (!ins->eof());
   
    return result;
  }

  void doClose() {
    sock.close();
    sock = NULL;
  }

private:

 Socket sock;

};

}

typedef Akenti::HttpURLHandlerImpl HUHI;

HUHI::HttpURLHandlerImpl() : URLHandlerImpl() {
}

HUHI::~HttpURLHandlerImpl() {
}

Akenti::URLConnection 
HUHI::getURLConnection(const URL& url) const {
  return URLConnection(new HttpURLConnectionImpl(url));
}
