#include "OpenSSLHmacImpl.hpp" 

typedef Akenti::OpenSSLHmacImpl OSHI;

OSHI::OpenSSLHmacImpl(const string& algo) : HmacImpl(algo) {
  if (!EVP_get_digestbyname(algo.c_str())) {
    throw Error("OSHI::OpenSSLHmacImpl:alg not supported");
  }
}

OSHI::~OpenSSLHmacImpl() {
}

int 
OSHI::getSize() const {
  return EVP_MD_size(EVP_get_digestbyname(alg.c_str()));
}

void 
OSHI::resetEngine() {
  HMAC_Init(
    &ctx, 
    key.getBytes(), 
    key.getSize(), 
    EVP_get_digestbyname(alg.c_str())
  );
}

void 
OSHI::doUpdate(const string& data, int offset, int len) {
  HMAC_Update(
    &ctx, 
    (unsigned char*) data.data() + offset, 
    len 
  );
}

string 
OSHI::doDigest() {
  string tag(getSize(), '\0');
  
  HMAC_Final(&ctx, (unsigned char*) tag.data(), NULL);
  return tag;
}
