#include "OpenSSLKeyImpl.hpp" 
#include "SecurityException.hpp" 
#include "Error.hpp" 

using Akenti::OpenSSLKeyImpl;

OpenSSLKeyImpl::OpenSSLKeyImpl(EVP_PKEY *evp_arg) : evp(evp_arg) {
  if (!evp_arg) {
    throw Error("OpenSSLKeyImpl:constructor:invalid arg");
  }
}
  
OpenSSLKeyImpl::~OpenSSLKeyImpl() {
  EVP_PKEY_free(evp);
}

int 
OpenSSLKeyImpl::size() const {
  return EVP_PKEY_size(evp);
}

string 
OpenSSLKeyImpl::algorithm() const {
  return "RSA";
}

string 
OpenSSLKeyImpl::format() const {
  return "DER";
}

string 
OpenSSLKeyImpl::privateEncrypt(const string &data) const {
  int len = size();
  string encrypted(len, '\0');

  int numbytes = RSA_private_encrypt(
    data.length(),
    (unsigned char*) data.data(),
    (unsigned char*) encrypted.data(),
    evp->pkey.rsa,
    RSA_PKCS1_PADDING
  );
 
  if (numbytes != len) {
    throw SecurityException("OpenSSLRSAPrivateKeyImpl:encrypt:encryption failed");
  }

  return encrypted;
}

string 
OpenSSLKeyImpl::privateDecrypt(const string &encrypted) const {
  int len = size();
  string data(len, '\0');

  int numbytes = EVP_PKEY_decrypt(
    (unsigned char*) data.data(),
    (unsigned char*) encrypted.data(),
    encrypted.length(),
    evp
  );

  if (numbytes == -1) {
    throw SecurityException("OpenSSLRSAPrivateKeyImpl:decrypt:decryption failed");
  }

  return data.substr(0, numbytes);
}

string 
OpenSSLKeyImpl::publicEncrypt(const string &data) const {
  int len = size();
  string encrypted(len, '\0');

  int numbytes = EVP_PKEY_encrypt(
    (unsigned char*) encrypted.data(),
    (unsigned char*) data.data(),
    data.length(),
    evp
  );

  if (numbytes != len) {
    throw SecurityException("OpenSSLRSAPublicKeyImpl:encrypt:encryption failed");
  }

  return encrypted;
}

string 
OpenSSLKeyImpl::publicDecrypt(const string &encrypted) const {
  int len = size();
  string data(len, '\0');

  len = RSA_public_decrypt(
    encrypted.length(),
    (unsigned char*) encrypted.data(),
    (unsigned char*) data.data(),
    evp->pkey.rsa,
    RSA_PKCS1_PADDING
  );

  if (len == -1) {
    throw SecurityException("OpenSSLRSAPublicKeyImpl:decrypt:decryption failed");
  }

  return data.substr(0, len);
}
