#include <openssl/pkcs12.h>
#include "PKCS12KeyStoreImpl.hpp" 
#include "OpenSSLPrivateKeyImpl.hpp" 
#include "OpenSSLX509CertificateImpl.hpp" 
#include "IOException.hpp" 
#include "SecurityException.hpp" 

typedef Akenti::PKCS12KeyStoreImpl PKSI;

PKSI::PKCS12KeyStoreImpl(const string& type) : KeyStoreImpl(type) {
}

PKSI::~PKCS12KeyStoreImpl() {
}
 
void 
PKSI::load(const string& fileName, const string& password) {
  BIO* bio = BIO_new_file((char *) fileName.c_str(), "r");
  
  if (!bio) {
    throw IOException(
      "OpenSSLKeyFactoryFactoryImpl:load:can't open file " + fileName
    );
  }

  PKCS12 *p12 = d2i_PKCS12_bio(bio, NULL);
  BIO_free(bio);

  if (!p12) {
    throw SecurityException(
      "PKCS12KeyStoreImpl:load:bad encoding"
    );
  }

  EVP_PKEY *evp = NULL;
  X509* cert = NULL;
  STACK *cas = NULL;
  
  if (PKCS12_parse(p12, (char*) password.c_str(), &evp, &cert, &cas) != 1) {
    PKCS12_free(p12);

    throw SecurityException(
      "PKCS12KeyStoreImpl:load:bad password "
    );
  }

  PKCS12_free(p12);

  privateKey = new OpenSSLPrivateKeyImpl(evp);
  subjectCert = new OpenSSLX509CertificateImpl(cert);
  
  for (int i = 0, n = sk_X509_num(cas); i < n; i++) {
    X509 *x = sk_X509_value(cas, i);

    x->references = 2;
    caCerts.push_back(new OpenSSLX509CertificateImpl(x));
  }

  sk_X509_free(cas);
}
