#ifndef AK_CIPHER_IMPL_HPP
#define AK_CIPHER_IMPL_HPP 

#include "Error.hpp"
#include "SecretKey.hpp"

namespace Akenti {

/**
  * @see SecretKey
  * @see Cipher
  * @author Abdelilah Essiari
  */
class CipherImpl {

public:

   /**
     * Destroys this cipher.
     */
   virtual ~CipherImpl() {
   }

   /**
     */
   void encryptInit(const SecretKey& key_arg) {
     state = ENCRYPTION;
     this->key = key_arg;
     resetEngine();
   }

   void decryptInit(const SecretKey& key_arg) {
     state = DECRYPTION;
     this->key = key_arg;
     resetEngine();
   }

   /**
     */
   string update(const string& data) {
     return update(data, 0, data.length());
   }

   /**
     */
   string update(const string& data, int offset, int len) {
     if (data.length() - offset < len) {
       throw Error("CipherImpl:update:input buffer too short");
     } else if(state == UNINITIALIZED) {
       throw Error("CipherImpl:update:invalid state");
     }
     
     return doUpdate(data, offset, len);
   }

   /**
     */
   string doFinal() {
     string ret = doDoFinal();
     
     state = UNINITIALIZED;
     return ret;
   }

protected:

   friend class Cipher;

   /**
     */
   virtual void resetEngine() {
   } 

   /**
     */
   virtual string doUpdate(const string& data, int offset, int len) {
     return data.substr(offset, len);
   } 

   /**
     */
   virtual string doDoFinal() {
     return "";
   }

   /**
     */
   CipherImpl(const string& alg) : alg(alg), state(UNINITIALIZED) {
   }

   enum State { UNINITIALIZED, ENCRYPTION, DECRYPTION } state;

   string alg;

   SecretKey key;
};

}

#endif
