#ifndef AKENTI_CONSTANTS_HPP
#define AKENTI_CONSTANTS_HPP

namespace Akenti {

  /**
    * Akenti version number.
    */
  static const char* CURRENT_VERSION = "1.5";
  static const char* SUPPORTED_VERSIONS[] = { CURRENT_VERSION, 0};

  /**
    * This version's namespaces.
    */
  static const char* CURRENT_PROT_NAME_SPACE = "http://dsd.lbl.gov/Akenti/2004/05/AkentiProtocol";
  static const char* CURRENT_CERT_NAME_SPACE = "http://dsd.lbl.gov/Akenti/2004/05/AkentiCertificate";
  static const char* CURRENT_SOAP_NAME_SPACE = "http://schemas.xmlsoap.org/soap/envelope/";

  /**
    * Maps supported namespaces to schema locations.
    *
    * If the location is not a <i>url</i> or an absolute path
    * then we locate the schemas in $(AKENTI_HOME)/xml/schema.
    */
  static const char* NAME_SPACES_MAP[][2] = { 
    { CURRENT_PROT_NAME_SPACE, "AkentiProtocol.xsd" },
    { CURRENT_SOAP_NAME_SPACE, "http://schemas.xmlsoap.org/soap/envelope/"},
    { 0, 0}
  };

  // BELOW ARE CONFIGURATION PROPERTIES

  /**
    * This version defaults to <i>XML</i>.
    *
    * Note: Previous versions to 1.5 should set this to <i>PEM</i>.
    */
  static const char* AK_CERT_ENC_TYPE = "XML";  // XML, PEM, or leave blank for both

  /**
    * Validation of xml certificates, requests, and responses defaults to <i>on</i>.
    */
  static const char* SchemaValidation = "on";

  /**
    * Policy certificate file names and resource info file names.
    */ 
  static const char* PolicyFileName = "htauthority";
  static const char* ResAttrFileName = "resattrs.xml";

  /**
    * SSLMode defaults to on.
    */
  static const char* SSLMode = "on";       // on or off
 
  /**
    * SSL params.
    */
  static const char* ServerCertFile  = "./server.pem";
  static const char* ServerKeyFile   = "./server.key";
  static const char* CertificateAuthorityPath = ".";
  static const char* CertificateAuthorityFile = "cas.pem";

  /**
    * Cache configuration.
    */
  static const char* Caching = "on";       // on or off
  static const char* CacheType = "file";   // file or server 

  /**
    * Caching to file system.
    */
  static const char* CacheDir = "./cache";       

  /**
    * Caching to server.
    */
  static const char* CacheHost = "localhost";
  static const char* CachePort = "10000";
}

#endif
