#include <iostream>

#include <cppunit/TextTestRunner.h>
#include "../shared/TestCaseConstants.hpp"
#include "EngComponentTestCase.hpp"
#include "Provider.hpp"
#include "OpenSSLProviderImpl.hpp"
#include "Properties.hpp"
#include "XMLUtil.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace Akenti;
using namespace std;

int
main(int ac, char **argv) {
  try {
    Provider provider = Provider(new OpenSSLProviderImpl);

    provider->loadDigestAlgorithms();
    provider->loadSignatureAlgorithms();
    Provider::setProvider(provider);
    XMLUtil::init();

    Properties& props = Properties::getSystemProperties();
    string confFile = CONF_DIR + CONF_FILE;

    props.load(confFile);
    props.setValue("Logging", "OFF");

    CppUnit::TextTestRunner runner;

    runner.addTest(EngComponentTestCase::suite());
    runner.run("", false);
    return 0;
  } catch(Exception& ex) {
    cout << "TestEngComponent:main" << ex.getMessage() << endl;
  } catch(Error& err) {
    cout << "TestEngComponent:main" << err.getMessage() << endl;
  }
}

