#ifndef CACHE_TEST_CASE_HPP   
#define CACHE_TEST_CASE_HPP

#include "cppunit/TestSuite.h"
#include "cppunit/TestCase.h"

/**
  * @author Abdelilah Essiari 
  * @author Jiewei Lin
  */
class CacheTestCase : public CppUnit::TestCase {

public:

  /**
    * Constructs this testcase.
    */ 
  CacheTestCase(const std::string& name = "CacheTestCase") 
    : TestCase(name) {
  }
    
  /**
    * Registers the <i>doTest</i> method.
    */
  static CppUnit::TestSuite* suite();

private:

  /**
    * Calls <i>runCacheTest1()</i>, <i>runCacheTest2()</i>,
    * <i>runCacheTest3()</i>, and <i>runCacheTest4()</i>.
    */
  void doTest();

  /**
    * Tests no duplicate and not verified certificates in CertificateCache
    * when caching is off.
    */ 
  void runCacheTest1();

  /**
    * Tests CertificateCache returns right level for each certificate it stores 
    * when caching is off. 
    */ 
  void runCacheTest2();

  /**
    * Tests CertificateCache updates external cache successfully. 
    */ 
  void runCacheTest3();

  /**
    * Tests CertificateCache does not update external cache if information was
    * loaded has not changed.
    */ 
  void runCacheTest4();

};

#endif
