#include <iostream>
#include "cppunit/TestCaller.h"
#include "cppunit/Exception.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "../shared/TestCaseConstants.hpp"
#include "AttributeCertGenerator.hpp"
#include "DistinguishedName.hpp"
#include "PrivateKey.hpp"
#include "KeyFactory.hpp"
#include "SecurityConstants.hpp"
#include "Exception.hpp"
#include "Error.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "CertificateResponseTestCase.hpp"
#include "CertificateInfo.hpp"
#include "AkentiMessage.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace std;

typedef CertificateResponseTestCase CRTC;

void 
CRTC::doTest() {
  string method;

  try {
    method = "CertificateResponseTestCase:doTest:testWithStatus:";
    testWithStatus();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
CRTC::testWithStatus() {
#if DEBUG
  cout << "Entering testWithStatus ..." << endl;
#endif

  AkentiCertificate cert = genAttributeCert();
  string msg(": Everything is fine");
  string location = "file:/some_location";
  AkentiMessage mesg(104, msg);
  vector<AkentiMessage> v(1, mesg);

  v.push_back(mesg);

  CertificateInfo cr1;

  cr1.setCertificate(cert);
  cr1.setCertificateMessages(v);
  cr1.setLocation(location);

  CPPUNIT_ASSERT(cert == cr1.getCertificate());

  DOMImplementation* impl = DOMImplementationRegistry::getDOMImplementation(XMLUtil::convert("Core"));
  DOMDocument* doc = impl->createDocument(0, XMLUtil::convert("CertificateInfo"), 0);
  DOMElement* element = doc->getDocumentElement();

  cr1.toXML(element);
#if DEBUG
  XMLUtil::write(cout, element);
#endif

  CertificateInfo cr2;

  cr2.fromXML(element);

  CPPUNIT_ASSERT(cr2.getCertificate() == cert);
  CPPUNIT_ASSERT(cr2.getCertificateMessages() == v);
  CPPUNIT_ASSERT(cr2.getLocation() == location);
  delete(doc);

#if DEBUG
  cout << "\nExiting testWithStatus ..." << endl;
#endif
}

CppUnit::TestSuite*
CRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("CertificateResponseTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<CertificateResponseTestCase> ( 
      "doTest", 
      &CertificateResponseTestCase::doTest
    )
  );

  return testSuite;
}

AkentiCertificate 
CRTC::genAttributeCert() {
#if DEBUG
  cout << "\n Generating Attribute Certificate ..." << endl;
#endif

  UtcTime now;
  UtcTime after(365 * 24 * 60 * 60);

  DistinguishedName ca(CAIA_DN);
  DistinguishedName issuer(SHI_DN);
  AkentiPrincipal guarantor(issuer, ca);

  DistinguishedName user(USERI_DN);
  AkentiPrincipal principal(user, ca);

  AttributeCertGenerator gen;

  gen.setVersionNumber(2);
  gen.setUniqueID("I");
  gen.setNotBefore(now);
  gen.setNotAfter(after);
  gen.setIssuer(guarantor);
  gen.setAttribute("JOB");
  gen.setValue("SUPERVISOR");
  gen.setPrincipal(principal);

  KeyFactory kfac = KeyFactory::getInstance();
  PrivateKey privKey = kfac->initPrivateKeyFromFile("PEM", ID_DIR + SHI_KEY);

  gen.sign(privKey, SecurityConstants::RSA_MD5);

  AkentiCertificate cert = gen.generate();

  return cert;
}
