#include <fstream>
#include <iostream>
#include <sstream>
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "cppunit/Exception.h"
#include "cppunit/TestCaller.h"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "ResourceBrowseResponseTestCase.hpp"
#include "ResourceBrowseResponse.hpp"
#include "AkentiResponses.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace std;

typedef ResourceBrowseResponseTestCase RBRTC;

void 
RBRTC::doTest() {
  string method;

  try {
    method = "ResourceBrowseResponseTestCase:doTest:test1:";
    test1();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
RBRTC::test1() {
#if DEBUG
  cout << "Entering test1! ..." << endl;
#endif

  vector<string> resources;

  resources.push_back("dir1");
  resources.push_back("dir2");

  ResourceBrowseResponse resp;

  resp.setResources(resources);
  CPPUNIT_ASSERT(resources == resp.getResources());

  AkentiResponses resps;

  resps.addResponse(resp);

  DOMElement* element = resps.toXML();
#if DEBUG
  XMLUtil::write(cout, element);
#endif

  resps.fromXML(element);
  delete(element->getOwnerDocument());
  resp = (ResourceBrowseResponse&) resps.getResponseAt(0);
  CPPUNIT_ASSERT(resources == resp.getResources());
#if DEBUG
  cout << "\nExiting test1 ..." << endl;
#endif
}

CppUnit::TestSuite*
RBRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("ResourceBrowseResponseTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<ResourceBrowseResponseTestCase> ( 
      "doTest", 
      &ResourceBrowseResponseTestCase::doTest
    )
  );

  return testSuite;
}
