#include <fstream>
#include <iostream>
#include <sstream>
#include "cppunit/TestCaller.h"
#include "cppunit/Exception.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "Exception.hpp"
#include "Error.hpp"
#include "SoapHttpResponseTestCase.hpp"
#include "SoapHttpResponse.hpp"
#include "SoapConstants.hpp"
#include "InputStream.hpp"
#include "OutputStream.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"

using namespace Akenti;
using namespace std;

typedef SoapHttpResponseTestCase SHRTC;

void
SHRTC::doTest() {
  string method;

  try {
    method = "SoapHttpResponseTestCase:doTest:test1:";
    test1();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
SHRTC::test1() {
#if DEBUG
  cout << "Entering test1 ..." << endl;
#endif

  DOMImplementation* impl = DOMImplementationRegistry::getDOMImplementation(XMLUtil::convert("Core"));
  DOMDocument* doc = impl->createDocument(0, XMLUtil::convert("SoapHttpResponse"), 0);
  DOMElement *retVal = doc->getDocumentElement();

  int statusCode = 200;
  string message("Gammasphere Status OK");
  string protocol("HTTP");
  string contentType("text/xml");
  SoapFaultCode faultCode =  NONE;

  SoapHttpResponse shr;
  
  shr.setStatusCode(statusCode);
  shr.setStatusMessage(message);
  shr.setProtocol(protocol);
  shr.setContentType(contentType);
  shr.setReturnValue(retVal);
  shr.setFaultCode(faultCode);

  ostringstream *oss = new ostringstream;
  OutputStream os(oss);

  shr.toStream(os);

  istringstream *iss = new istringstream(oss->str());
  InputStream is(iss);

  SoapHttpResponse temp;

  temp.fromStream(is);

  CPPUNIT_ASSERT(statusCode == shr.getStatusCode());
  CPPUNIT_ASSERT(message == shr.getStatusMessage());
  CPPUNIT_ASSERT(protocol == shr.getProtocol());
  CPPUNIT_ASSERT(contentType == shr.getContentType());
  CPPUNIT_ASSERT(faultCode == shr.getFaultCode());
  CPPUNIT_ASSERT(retVal->isEqualNode(shr.getReturnValue()));

  CPPUNIT_ASSERT(statusCode == temp.getStatusCode());
  CPPUNIT_ASSERT(message == temp.getStatusMessage());
  CPPUNIT_ASSERT(protocol == temp.getProtocol());
  CPPUNIT_ASSERT(contentType == temp.getContentType());

  CPPUNIT_ASSERT(shr.getFaultMessage() == temp.getFaultMessage());
  CPPUNIT_ASSERT(faultCode == temp.getFaultCode());
#if DEBUG
  cout << "Exiting test1" << endl;
#endif
}

CppUnit::TestSuite*
SHRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("SoapHttpResponseTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<SoapHttpResponseTestCase> ( 
      "doTest", 
      &SoapHttpResponseTestCase::doTest
    )
  );

  return testSuite;
}
