#include <fstream>
#include <iostream>
#include <sstream>
#include "cppunit/TestCaller.h"
#include "cppunit/Exception.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "DistinguishedName.hpp"
#include "PrivateKey.hpp"
#include "KeyFactory.hpp"
#include "Util.hpp"
#include "SecurityConstants.hpp"
#include "../shared/TestCaseConstants.hpp"
#include "File.hpp"
#include "CertificateFactory.hpp"
#include "PolicyCertGenerator.hpp"
#include "AkentiRequests.hpp"
#include "Exception.hpp"
#include "Error.hpp"
#include "Util.hpp"
#include "UploadCertificateRequestTestCase.hpp"
#include "UploadPolicyCertificateRequest.hpp"

using namespace std;

typedef UploadCertificateRequestTestCase UCRTC;

void 
UCRTC::doTest() {
  string method;

  try {
    method = "UploadCertificateRequestTestCase:doTest:test1:";
    test1();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
UCRTC::test1() {
#if DEBUG
  cout << "Entering test1 ..." << endl;
#endif
  AkentiCertificate tempCert = genPolicyCert();
  PolicyCertificate cert = (PolicyCertificate&) tempCert;
  UploadPolicyCertificateRequest ucr;

  ucr.setCertificate(cert);
  CPPUNIT_ASSERT(cert == ucr.getCertificate());

  AkentiRequests reqs;

  reqs.addRequest(ucr);

  DOMElement* element = reqs.toXML();
  AkentiRequests temp;

  temp.fromXML(element);
  delete(element->getOwnerDocument());
  ucr = (UploadPolicyCertificateRequest&) temp.getRequestAt(0);
  CPPUNIT_ASSERT(cert == ucr.getCertificate());
#if DEBUG
  cout << reqs.toString() << endl;
  cout << "\nExiting test1 ..." << endl;
#endif
}

CppUnit::TestSuite*
UCRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("UploadCertificateRequestTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<UploadCertificateRequestTestCase> ( 
      "doTest", 
      &UploadCertificateRequestTestCase::doTest
    )
  );

  return testSuite;
}

AkentiCertificate 
UCRTC::genPolicyCert() {
#if DEBUG
  cout << "\nGenerating ROOT3 Policy Certificate ..." << endl;
#endif 

  UtcTime now;
  UtcTime after(365 * 24 * 60 * 60);

  DistinguishedName ca(CAIA_DN);
  DistinguishedName issuer(SHI_DN);
  AkentiPrincipal guarantor(issuer, ca);

  Directory idDirectory("file:" + File(ID_DIR).getAbsolutePath());
  vector<Directory> idDirectories(1, idDirectory);

  Directory attrDirectory("file:" + File(AKCERT_DIR).getAbsolutePath());
  vector<Directory> attrDirectories(1, attrDirectory);

  Directory ucDirectory("file:" + File(AKCERT_DIR).getAbsolutePath());
  vector<Directory> ucDirectories(1, ucDirectory);

  vector<AkentiPrincipal> ucIssuers(1, guarantor);
  UseCondIssuerGroup group(ucIssuers, ucDirectories);
  vector<UseCondIssuerGroup> groups(1, group);

  CertificateFactory cfac = CertificateFactory::getInstance();
  vector<Certificate> trustedCAs = cfac->initCertificatesFromFile(
    SecurityConstants::PEM, ID_DIR + CAI_CERT 
  );
  CertificateAuthorityInfo info(trustedCAs[0], idDirectories);
  vector<CertificateAuthorityInfo> infos(1, info);

  PolicyCertGenerator gen;

  gen.setVersionNumber(2);
  gen.setUniqueID("1");
  gen.setNotBefore(now);
  gen.setNotAfter(after);
  gen.setCacheTime(30 * 60);
  gen.setResource(ROOT3);
  gen.setIssuer(guarantor);
  gen.setAttributeDirectories(attrDirectories);
  gen.setUseCondIssuerGroups(groups);
  gen.setCertificateAuthorityInfos(infos);

  KeyFactory kfac = KeyFactory::getInstance();

  PrivateKey privKey = kfac->initPrivateKeyFromFile(
    SecurityConstants::PEM, ID_DIR + SHI_KEY
  );

  gen.sign(privKey, SecurityConstants::RSA_MD5);

  AkentiCertificate cert = gen.generate();

  return cert;
}
