#include <cppunit/Exception.h>
#include "cppunit/TestCaller.h"
#include "cppunit/TextTestRunner.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "Base64TestCase.hpp"

#include "Base64.hpp"

using namespace Akenti;
using namespace std;

void 
Base64TestCase::doTest() {
  string method;

  try {
    method = "Base64TestCase::test1:";
    test1();
    method = "Base64TestCase::test2:";
    test2();
  } catch(Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch(Error& err) {
    CppUnit::Message m(method + err.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void 
Base64TestCase::test1() {
  string data   = "some other characters ^ @ # $ 80 "; 

  CPPUNIT_ASSERT(data == Base64::decode(Base64::encode(data)));
}

void
Base64TestCase::test2() {
  Base64 encoder = Base64::getInstance();

  encoder->encodeInit();

  string data = "some other characters ^ @ # $ 80 "; 

  encoder->encodeUpdate(data);

  string encoding = encoder->encode();

  Base64 decoder = Base64::getInstance();

  decoder->decodeInit();
  decoder->decodeUpdate(encoding);
  CPPUNIT_ASSERT(data == decoder->decode());
}

CppUnit::TestSuite* 
Base64TestCase::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("Base64TestCase");

  testSuite->addTest(
    new CppUnit::TestCaller<Base64TestCase>(
      "doTest", 
      &Base64TestCase::doTest
    )
  );

  return testSuite;
}
