#include <cppunit/Exception.h>
#include "cppunit/TestCaller.h"
#include "cppunit/TextTestRunner.h"
#include "KeyPairTestCase.hpp"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "KeyPair.hpp"
#include "KeyFactory.hpp"
#include "Base64.hpp"

using namespace Akenti;

void 
KeyPairTestCase::doTest() {
  string method;

  try {
    method = "KeyPairTestCase::generateRSAKeyPair:";
    generateRSAKeyPair();
    method = "KeyPairTestCase::generateDSAKeyPair:";
    generateDSAKeyPair();
  } catch(Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch(Error& err) {
    CppUnit::Message m(method + err.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
KeyPairTestCase::generateRSAKeyPair() {
  KeyFactory fac = KeyFactory::getInstance();
  KeyPair pair = fac->generateRSAKeyPair();

  string enc1 = fac->getKeyEncoded("DER", pair.getPublicKey());
  string enc2 = fac->getKeyEncoded("DER", pair.getPrivateKey());

  PublicKey publicKey = fac->initPublicKey("DER", enc1);
  PrivateKey privateKey = fac->initPrivateKey("DER", enc2);
}

void
KeyPairTestCase::generateDSAKeyPair() {
  KeyFactory fac = KeyFactory::getInstance();
  KeyPair pair = fac->generateDSAKeyPair();

  string enc1 = fac->getKeyEncoded("DER", pair.getPublicKey());
  string enc2 = fac->getKeyEncoded("DER", pair.getPrivateKey());

  PublicKey publicKey = fac->initPublicKey("DER", enc1);
  PrivateKey privateKey = fac->initPrivateKey("DER", enc2);
}

CppUnit::TestSuite* 
KeyPairTestCase::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("KeyPairTestCase");

  testSuite->addTest(
    new CppUnit::TestCaller<KeyPairTestCase>( 
      "doTest", 
      &KeyPairTestCase::doTest
    )
  );
  
  return testSuite;
}
