#include <sstream>
#include <cppunit/Exception.h>
#include "cppunit/TestCaller.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "MessageDigestTestCase.hpp"
#include "MessageDigest.hpp"
#include "SecurityConstants.hpp"

using namespace Akenti;
using namespace std;

static char
hexascii(int h) {
  if (h >= 10 && h <= 15) {
    return (char)('a' + (h - 10));
  }

  if (h >= 0 && h <= 9) {
    return (char)('0' + h);
  }
}

static string 
toHex(string bin) {
  string hex;

  for (int j = 0; j < bin.length(); j++) {
    hex.append(1, hexascii(bin[j] >> 4 & 0x0F));
    hex.append(1, hexascii(bin[j] & 0x0F));
  }

  return hex;
}

void 
MessageDigestTestCase::doTest() {
  string method;

  try {
    method = "MessageDigestTestCase:testMD5:";
    testMD5();
    method = "MessageDigestTestCase:testSHA1:";
    testSHA1();
  } catch(Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch(Error& err) {
    CppUnit::Message m(method + err.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
MessageDigestTestCase::testMD5() {   
  string data = "message digest";
  string expectedHash = "f96b697d7cb7938d525a2f31aaf161d0";

  CPPUNIT_ASSERT(toHex(MessageDigest::digest(SecurityConstants::MD5, data)) == expectedHash);
}

void 
MessageDigestTestCase::testSHA1() {
  string data = "message digest";
  string expectedHash = "c12252ceda8be8994d5fa0290a47231c1d16aae3";

  MessageDigest md = MessageDigest::getInstance(SecurityConstants::SHA1);        

  md->update("JUNK JUNK JUNK");
  md->reset();
  md->update(data, 0, data.length() / 2);
  md->update(data, data.length() / 2, data.length() - data.length() / 2);
  CPPUNIT_ASSERT(toHex(md->digest()) == expectedHash);
}

CppUnit::TestSuite*
MessageDigestTestCase::suite() {
  CppUnit::TestSuite *testSuite = new CppUnit::TestSuite("MessageDigestTestCase");     

  testSuite->addTest(
    new CppUnit::TestCaller<MessageDigestTestCase>(
      "doTest",  
      &MessageDigestTestCase::doTest
    )
  );       

  return testSuite;
}
