#include <cppunit/Exception.h>
#include "cppunit/TestCaller.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"

#include "SignatureTestCase.hpp"
#include "Signature.hpp"
#include "KeyFactory.hpp"
#include "KeyPair.hpp"
#include "Base64.hpp"

using namespace Akenti;

void 
SignatureTestCase::doTest() {
  string method;

  try {
    method = "SignatureTestCase:signatureRSAWithMD5:";
    signatureRSAWithMD5();
    method = "SignatureTestCase:signatureDSAWithSHA1:";
    signatureDSAWithSHA1();
  } catch(Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch(Error& err) {
    CppUnit::Message m(method + err.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
SignatureTestCase::signatureRSAWithMD5(){
  KeyFactory fac = KeyFactory::getInstance();
  KeyPair pair = fac->generateRSAKeyPair();
  Signature sig = Signature::getInstance("RSA-MD5");

  sig->initSign(pair.getPrivateKey());
  sig->update("HELLO TO YOU MY FRIEND");

  string signature = sig->sign();

  sig->initVerify(pair.getPublicKey());
  sig->update("HELLO ");
  sig->update("TO YOU MY FRIEND");
  CPPUNIT_ASSERT(sig->verify(signature));
}

void
SignatureTestCase::signatureDSAWithSHA1(){
  KeyFactory fac = KeyFactory::getInstance();
  KeyPair pair = fac->generateDSAKeyPair();
  Signature sig = Signature::getInstance("DSA-SHA1");

  sig->initSign(pair.getPrivateKey());
  sig->update("HELLO TO YOU MY FRIEND");

  string signature = sig->sign();

  sig->initVerify(pair.getPublicKey());
  sig->update("HELLO TO ");
  sig->update("YOU MY FRIEND");
  CPPUNIT_ASSERT(sig->verify(signature));
}

CppUnit::TestSuite* 
SignatureTestCase::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("SignatureTestCase");

  testSuite->addTest(
    new CppUnit::TestCaller<SignatureTestCase>(
      "doTest", 
      &SignatureTestCase::doTest
    )
  );

  return testSuite;
}
