#ifndef TEST_CASE_CONSTANTS_HPP
#define TEST_CASE_CONSTANTS_HPP

#include <string>

using std::string;

static const string AK_CERT_ENC_TYPE = "XML"; 

/**
  * Directory where the test will be installed.
  */
static const string BASE_DIR = "../../resourceTree/";

/**
  * Configuration file used by the akenti engine.
  */
static const string CONF_DIR = BASE_DIR + "conf/"; 
static const string CONF_FILE = "akentiConf"; 


/**
  * Directory where X509 certificates  will be stored. 
  * 
  * Note: The engine will use this directory to look up these
  * certificates during the authorization process.
  */
static const string ID_DIR = BASE_DIR + "idCerts/"; 

/**
  * Directory where attribute and use condition certificates
  * will be stored. 
  *
  * Note: The engine will use this directory to look up these
  * certificates during the authorization process.
  */
static const string AKCERT_DIR = BASE_DIR + "akentiCerts/"; 

/**
  * Directory where the engine will cache X509, UseCond, attribute 
  * and capability certificates.
  */
static const string CACHE_DIR = BASE_DIR + "cache";

/**
  * File where the engine will store the logs if the file
  * option is enabled. 
  */
static const string LOG_DIR = BASE_DIR + "logs/"; 
static const string LOG_FILE = "testLogs"; 
 
/**
  * TESTCA I info.
  *
  * This entity will issue X509 certificates for TESTCA IA and TESTCA IB.
  */
static const string CAI_DN = "/C=US/OU=LBL/CN=TESTCA I";
static const string CAI_CERT = "caICert";
static const string CAI_KEY = "caIKey";

/**
  * TESTCA IA info.
  *
  * This entity will issue X509 certificates for
  * Stake Holder I, and User I.
  */
static const string CAIA_DN = "/C=US/OU=LBL/CN=TESTCA IA";
static const string CAIA_CERT = "caIACert";
static const string CAIA_KEY = "caIAKey";

/**
  * Authorization Server Info.  
  *
  * This entity issues all capability certificates
  * if security option is on.
  */
static const string SERVER_DN = "/C=US/OU=LBL/CN=TESTSERVER";
static const string SERVER_CERT = "serverCert";
static const string SERVER_KEY = "serverKey";

/**
  * Client info.  
  *
  * This entity plays the role of a resource provider.
  *
  */
static const string CLIENT_CONF_FILE = "clientConf"; 
static const string CLIENT_DN = "/C=US/OU=LBL/CN=TESTCLIENT";
static const string CLIENT_CERT = "clientCert";
static const string CLIENT_KEY = "clientKey";

/**
  * Akenti Issuer Stake Holder I Info.
  *
  * This entity will issue some policy, attribute, and use-condition
  * certificates.
  */
static const string SHI_DN = "/C=US/OU=LBL/CN=Stake Holder I";
static const string SHI_CERT = "shICert";
static const string SHI_KEY = "shIKey";

/**
  * User I Info.
  *
  * This entity will issue the prxy certificate.
  */
static const string USERI_DN = "/C=US/OU=LBL/CN=User I";
static const string USERI_CERT = "userICert";
static const string USERI_KEY = "userIKey";

/**
  * Proxy Info.
  */
static const string PROXY_DN = "/C=US/OU=LBL/CN=User I/CN=proxy"; 
static const string PROXY_CERT = "proxyCert"; 
static const string PROXY_KEY = "proxyKey"; 

/**
  * TESTCA IB info.
  *
  * This entity will issue X509 certificates for User IB1, 
  * User IB2, and Stake Holder IB.
  *
  */
static const string CAIB_DN = "/C=US/OU=LBL/CN=TESTCA IB";
static const string CAIB_CERT = "caIBCert";
static const string CAIB_KEY = "caIBKey";

/**
  * User IB1 Info.
  *
  */
static const string USERIB1_DN = "/C=US/OU=LBL/CN=User IB1";
static const string USERIB1_CERT = "userIB1Cert";
static const string USERIB1_KEY = "userIB1Key";

/**
  * User IB2 Info.
  *
  */
static const string USERIB2_DN = "/C=US/OU=LBL/CN=User IB2";
static const string USERIB2_CERT = "userIB2Cert";
static const string USERIB2_KEY = "userIB2Key";

/**
  * Akenti Issuer Stake Holder IB Info.
  *
  * This entity will issue some policy, attribute, and use-condition
  * certificates.
  */
static const string SHIB_DN = "/C=US/OU=LBL/CN=Stake Holder IB";
static const string SHIB_CERT = "shIBCert";
static const string SHIB_KEY = "shIBKey";

/**
  * The server will protect two root resources, R1, R2 and R3.
  *
  * <i>R1</i> is a flat resource.
  * <i>R2</i> a hierchical resource.
  * <i>R3</i> is a flat resource.
  */
static const string ROOT1 = "R1";
static const string ROOT1_DIR = BASE_DIR + ROOT1 + "/";

/**
  * A hierchical resource </i>R2/S1/S2/S3/S4</i>.
  * 
  * The resource <i>R2</i> is the root resource.
  */
static const string ROOT2 = "R2";
static const string ROOT2_DIR = BASE_DIR + ROOT2 + "/";

static const string SUB1 = ROOT2 + "/S1";
static const string SUB1_DIR = ROOT2_DIR + "S1/";

static const string SUB2 = SUB1 + "/S2";
static const string SUB2_DIR = SUB1_DIR + "S2/";

static const string SUB3 = SUB2 + "/S3";
static const string SUB3_DIR = SUB2_DIR + "S3/";

static const string SUB4 = SUB3 + "/S4"; 
static const string SUB4_DIR = SUB3_DIR + "S4/"; 

/**
  *<i>R3</i> is s flat resource. 
  */
static const string ROOT3 = "R3";
static const string ROOT3_DIR = BASE_DIR + ROOT3 + "/";


/**
  *<i>R4</i> is s flat resource. 
  */
static const string ROOT4 = "R4";
static const string ROOT4_DIR = BASE_DIR + ROOT4 + "/";

/**
  * Name of file used to store policy certificates.
  */
static string POLICY_FILE_NAME = "htauthority";

#endif
