/**
  * Main that will run all the test suites for the classes under the 
  * shared directory 
  *  
  * usage  TestShared <name_of_the_class>, where name_of_the_class is the name 
  * of the class that has been set up with the unit tests and have a suite 
  * method that contains all the unit tests.  
  */
#include <cppunit/TextTestRunner.h>
#include <cppunit/TextTestResult.h>
#include <cppunit/Exception.h>
#include <cppunit/Message.h>
#include <cppunit/SourceLine.h>
#include "Provider.hpp"
#include "OpenSSLProviderImpl.hpp"
#include "InstallResourceTree.hpp"
#include "CreateAkentiConf.hpp"
#include "X509CertTestCase.hpp"
#include "Base64TestCase.hpp"
#include "MessageDigestTestCase.hpp" 
#include "SignatureTestCase.hpp" 
#include "KeyPairTestCase.hpp" 
#include "OpenSSLProviderImpl.hpp"
#include "URLTestCase.hpp"
#include "StreamTokenizerTestCase.hpp"
#include "DistinguishedNameTestCase.hpp"

using namespace Akenti;
using namespace std;
using namespace CppUnit;

int 
main(int ac, char **argv) {
  try {

    Provider po = Provider(new OpenSSLProviderImpl);   

    po->loadDigestAlgorithms();
    po->loadSignatureAlgorithms();

    bool seeded = po->seedRandomGenerator(
      "WE need to seed the random generator"
    );

    if (!seeded) {
      throw SecurityException("TestShared:main:Unable to seed generator");
    }

    Provider::setProvider(po);
  } catch(SecurityException &ex) {
    CppUnit::Message m("TestShared:main: " + ex.getMessage());
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch(Error& err) {
    CppUnit::Message m("TestShared:main:FATAL:" + err.getMessage());
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }

  TextTestRunner runner;

  runner.addTest(DistinguishedNameTestCase::suite());
  runner.addTest(MessageDigestTestCase::suite());

  runner.addTest(InstallResourceTree::suite());
  runner.addTest(CreateAkentiConf::suite());
  runner.addTest(X509CertTestCase::suite());
  runner.addTest(Base64TestCase::suite());
  runner.addTest(SignatureTestCase::suite());
  runner.addTest(KeyPairTestCase::suite());
  runner.addTest(URLTestCase::suite());
  runner.addTest(StreamTokenizerTestCase::suite());
  runner.run("", false);
  return 0;
}
