#include "URLTestCase.hpp"

#include <cppunit/Exception.h>
#include "cppunit/TestCaller.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"

#include "URLConnection.hpp"
#include "URL.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace std;
using namespace Akenti;

void 
URLTestCase::doTest() {
  string method;

  try {
    method = "URLTestCase::parseURLS:";
    parseURLS();
    method = "URLTestCase::openHttpURL:";
    openHttpURL();
    method = "URLTestCase::openFileURL:";
    openFileURL();
  } catch(Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch(Error& err) {
    CppUnit::Message m(method + err.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
URLTestCase::parseURLS() {
  URL url1("file:/home/ex");
  URL url2("http://george.lbl.gov/home/ex");
  URL url3("http://portnoy.lbl.gov:8088");
}

void
URLTestCase::openHttpURL() {
  URL url("http://www-itg.lbl.gov/ImgLib/welcome.html");    
  URLConnection conn = url.getURLConnection();

  conn->connect();

  string content = conn->getContent();

  conn->close();
}

void
URLTestCase::openFileURL() {
  // XXX
}

CppUnit::TestSuite*
URLTestCase::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("URLTestCase");

  testSuite->addTest(
    new CppUnit::TestCaller<URLTestCase> (
      "doTest",
      &URLTestCase::doTest
    )
  );

  return testSuite;
}

