#include <sstream>
#include <iostream>
#include <string>
#include <unistd.h>
#include "UploadPolicyCertificateRequest.hpp"
#include "UploadPolicyCertificateResponse.hpp"
#include "PolicyCertificate.hpp"
#include "ProtocolHandler.hpp"
#include "AkentiInit.hpp"
#include "Exception.hpp"
#include "Error.hpp"
#include "File.hpp"

using namespace Akenti;
using namespace std;

/**
  * <pre>
  * <b>UploadPolicyCertificate</b>
  *   Use this program to upload a policy certificate.
  * </pre>
  */
namespace UploadPolicyCertificate {

void printPurpose() {
  cerr << "Use this program to upload a policy certificate." << endl 
       << endl;
}

void printUsage () {
  cerr << "Usage: UploadPolicyCertificate args" << endl
       << "  -f arg          - akenti engine config file" << endl
       << "  -p arg          - policy certificate file" << endl
       << "  [-v]            - verbose" << endl
       << "  [-h]            - prints this message" << endl;
}

int 
main(int argc, char** argv) {
  if (argc < 2) {
    printPurpose();
    printUsage();
    exit(2);
  }

  string confFileName;
  string policyFileName;
  bool verbose = false;
  int opt;

  while ((opt = getopt(argc,argv,"f:p:vh")) != EOF) {
    switch(opt){
      case 'f':
        confFileName = optarg;
        break;
      case 'p':
        policyFileName = optarg;
        break;
      case 'v':
        verbose = true;
        break;
      case 'h':
      default:
        printUsage();
        exit(2);
    }
  }

  try {
    if (verbose) {
      cout  << "Initializing Akenti engine ..." << endl;
    }

    string status;
  
    if (!AkentiInit::init(confFileName, status)) {
      cerr << status << endl;
      exit(2);
    }

    if (verbose) {
      cout  << status << endl;
    }

    if (policyFileName == "") {
      cerr << "Must provide a policy file name" << endl;
      exit(2);
    }
    File testFile(policyFileName);

    if (!testFile.exists() || !testFile.isFile()) {
      cerr << "Must provide a valid policy file" << endl;
      exit(2);
    }
 
    AkentiCertificate cert = AkentiCertificate::initFromFile(policyFileName);
    
    if (!cert.instanceOf<PolicyCertificate>()) {
      cerr << "Policy file does not contain a policy certificate" << endl;
      exit(2);
    }

    UploadPolicyCertificateRequest req;
    UploadPolicyCertificateResponse resp;
 
    req.setID(1);
    req.setCertificate((PolicyCertificate&) cert);
    ProtocolHandler::getInstance(ProtocolHandler::ENGINE).handle(req, resp);
   
    AkentiMessage mesg = resp.getAkentiMessage();

    cout << mesg.toString() << endl;

    exit(mesg.getCode() >= 200);
  } catch(Exception& ex) {
    cerr << "Exception:" << ex << endl;
    exit(2);
  } catch(Error& err) {
    cerr << "ERROR:Report to akenti team:" << err << endl;
    exit(3);
  } 
}

}
