#include "CacheCertificateImpl.hpp"
#include "AkentiOutputStream.hpp"
#include "AkentiInputStream.hpp"
#include "Exception.hpp"
#include "Error.hpp"
#include "Util.hpp"

using std::endl;

namespace Akenti {

typedef CacheCertificateImpl CCI;

CCI::CacheCertificateImpl() 
  : AkentiCertificateImpl("Cache") , holders() {
}

CCI::~CacheCertificateImpl() {
}

CacheCertificateImpl* 
CCI::newInstance() const {
  return new CacheCertificateImpl;
}

const vector<AKCertHolder>& 
CCI::getAKCertHolders() const {
  return holders;
}

void 
CCI::setAKCertHolders(const vector<AKCertHolder>& holders) {
  if (!holders.size()) {
    throw Error(
      "CacheCertificateImpl:setAKCertHolders:size = 0"
    );
  }

  string hashCode = holders[0]->getCertificate()->hashCode();

  for (int i = 1; i < holders.size(); ++i) {
    if (holders[i]->getCertificate()->hashCode() != hashCode) {
      throw Error(
        "CacheCertificateImpl:setAKCertHolders:illegal argument"
      );
    }
  }

  this->holders = holders;
}

void
CCI::doWrite(AkentiOutputStream& drain) const {
  int size = holders.size();

  if (!size) {
    throw Error(
      "CacheCertificateImpl:doWrite:size = 0"
    );
  }

  drain << size;
  
  for (int i = 0; i < size; ++i) {
  // changed from a single statement to many, because on linux drain was getting
  // set to 0 somewhere along the line. mrt/8-13-02 

    drain << ' ';  
    drain      << holders[i]->getCertificate(); 
    drain      << ' ';
    drain      << holders[i]->getLevel();
    drain      << ' ';
    drain      << holders[i]->getStartCacheTime();
    drain      << ' ';
    drain      << holders[i]->getEndCacheTime();
  }
}

void
CCI::doRead(AkentiInputStream& source) {
  holders.clear();

  int size;
 
  source >> size;
  
  if (size <= 0) {
    throw Exception(
      "CacheCertificateImpl:doRead:size <= 0"
    );
  }

  for (int i = 0; i < size; ++i) {
    AkentiCertificate cert;

    source >> cert; 

    int level;
  
    source >> level;
   
    UtcTime startCacheTime;
    
    source >> startCacheTime;
    
    UtcTime endCacheTime;

    source >> endCacheTime;

    AKCertHolder holder(new AKCertHolderImpl(
      cert, level, startCacheTime, endCacheTime
    ));

    holders.push_back(holder);
  }
}

string 
CCI::getHashString() const {
  if (!holders.size()) {
    throw Error(
      "CacheCertificateImpl:getHashString:size = 0"
    );
  }

  return holders[0]->getCertificate()->getHashString();
}

void
CCI::doParamString(ostream& os) const {
  vector<AKCertHolder> holders = getAKCertHolders();

  os <<  '[';
 
  if (holders.size()) {
    os << holders[0]->paramString();

    for (int i = 1; i < holders.size(); ++i) {
      os << ',' << holders[i]->paramString();
    }
  }

  os <<  ']';
}

void 
CCI::doToText(int indentAmt, ostream& os) const {
  const vector<AKCertHolder>& holders = getAKCertHolders(); 

  int size = holders.size();

  for (int i = 0; i < size; ++i) {  
    os << Util::fText(indentAmt + 2, " ") << i + 1 << '.' << endl;
    os << Util::fText(indentAmt + 3, "Validity Period for Cache Certificate: ")
       << endl; 
    os << Util::fText(indentAmt + 6, "Start Cache Time : ") 
       << holders[i]->getStartCacheTime().toString()
       << endl;

    os << Util::fText(indentAmt + 6, "End Cache Time   : ") 
       << holders[i]->getEndCacheTime().toString()
       << endl;
    os << holders[i]->getCertificate()->toText(indentAmt + 3) << endl;
  } 
} 

}
