#include "AkentiIssuer.hpp"
#include "AkentiOutputStream.hpp"
#include "AkentiInputStream.hpp"
#include "URL.hpp"
#include "Exception.hpp"
#include "XMLUtil.hpp"
#include "Xerces-Includes.hpp"

namespace Akenti {

typedef AkentiIssuer AI;

AI::AkentiIssuer(const AkentiPrincipal& issuer,
                 const vector<Directory>& idDirs)
  : issuer(issuer), idDirs(idDirs) {
}

AI::~AkentiIssuer() {
}

const AkentiPrincipal&
AI:: getIssuer() const {
  return issuer;
}

const DistinguishedName&
AI::getName() const {
  return issuer.getName();
}

const DistinguishedName&
AI::getGuarantor() const {
  return issuer.getGuarantor();
}

const vector<Directory>&
AI::getIdentityDirs() const {
  return idDirs;
}

bool 
AI::parseIdentityDirs(vector<URL>& urls) const {
  bool ret = true;

  for (int i = 0;i < idDirs.size(); ++i) {
    URL url;

    ret = ret ? idDirs[i].parseDir(url) : false;
    urls.push_back(url);
  }
  
  return ret;
}

bool
AI::operator==(const AkentiIssuer& issuer) const {
  return this->issuer == issuer.getIssuer();
}

void
AI::writeObject(AkentiOutputStream& drain) const {
  drain << issuer;

  int size = idDirs.size();

  drain << ' ' << size; 
  
  for (int i = 0; i < size; ++i) {
    drain << ' ' << idDirs[i];
  }
}

void 
AI::readObject(AkentiInputStream& source) {
  idDirs.clear();
  source >> issuer;

  int size;

  source >> size;
  
  if (size < 0) {
    throw Exception("AkentiIssuer:readObject:size < 0");
  }

  for (int i = 0; i < size; ++i) {
    Directory dir;

    source >> dir;
    idDirs.push_back(dir);
  }
}

void 
AI::toXML(DOMElement* issuerElement) const {
  issuer.toXML(issuerElement);
  
  int size = idDirs.size();

  if (!size) {
    return;
  }

  DOMDocument *doc = issuerElement->getOwnerDocument();

  for (int i = 0; i < size; ++i) {
    DOMElement *dir = doc->createElement(XMLUtil::convert("URL"));

    idDirs[i].toXML(dir);
    issuerElement->appendChild(dir);
  }
}

void 
AI::fromXML(const DOMElement* issuerElement) {
  idDirs.clear();
  issuer.fromXML(issuerElement);
  
  DOMNodeList *dirsList = issuerElement->getElementsByTagName(XMLUtil::convert("URL"));
 
  for (int i=0; i < dirsList->getLength(); i++) {
    DOMNode *temp = dirsList->item(i);
    DOMElement *dirElement = (DOMElement*) temp;
    Directory dir;

    dir.fromXML(dirElement);
    idDirs.push_back(dir);
  }
}

string
AI::paramString() const {
  string params;

  params = '[';
  params += issuer.paramString();
  params += ',';
  params += '[';

  for (int i = 0; i < idDirs.size(); ++i) {
    params += idDirs[i].paramString();
  }

  params += ']';
  params += ']';
  return params;
}

string
AI::toLogString() const {
  return issuer.toLogString();
}

} // end of namespace
