#ifndef CONDITION_HPP 
#define CONDITION_HPP 

#include <string>
#include <vector>

#include "AkentiObject.hpp"
#include "AttributeInfo.hpp"

using std::string; 
using std::vector; 

namespace Akenti {

class Expression;

/**
  * Holds a boolean expression and a vector of <i>AttributeInfo</i>
  * objects that describe the attribute/value pairs contained in 
  * the expression. 
  *
  * <pre>
  * Condition {
  *    string constraint; 
  *    vector&lt;AttributeInfo&gt; attributeInfos;
  * }
  * </pre>
  *
  * @see AttributeInfo
  * @see Expression
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

class Condition : public AkentiObject {

public:

  /** 
    * Default constructor.
    */
  Condition();
  
  /**
    */   
  Condition(const string& constraint, const vector<AttributeInfo>& attributes);

  /**
    * Destroys this object.
    */
  ~Condition();

  /**
    * Assignement operator.
    */
  void operator=(const Condition& condition);

  /**
    *
    */
  bool parseConstraint(Expression& expr) const;

  /**
    *
    */
  const string& getConstraint() const;

  /**
    */
  const vector<AttributeInfo>& getAttributeInfos() const;

  /**
    */
  bool isValid() const;

  /**
    */
  void writeObject(AkentiOutputStream& drain) const;

  /**
    */
  void readObject(AkentiInputStream& source);

  /**
    * Populates the XML element <i>cond</i> from this <i>Condition</i>.
    */
  void toXML(DOMElement* cond) const;

  /**
    * Initializes this <i>Condition</i> from the XML element
    * <i>cond</i>.
    */
  void fromXML(const DOMElement* cond);

  /**
    * Used for debugging.
    *
    * @return a string representation of this object
    *
    * @see AkentiObject
    */
  string paramString() const;

private:

  string constraint;

  vector<AttributeInfo> attributeInfos;

};

} // end of name space

#endif
