#include <fstream> 
#include "Policy.hpp"
#include "UseCondCertificateImpl.hpp"
#include "Error.hpp"

using namespace Akenti;

typedef PolicyImpl PCI;

PCI::PolicyImpl(bool leaf, const PolicyCertificate& policyCert) 
  : leaf(leaf), policyCert(policyCert) {
}

PCI::~PolicyImpl() {
}

bool
PCI::isLeaf() const {
  return leaf;
}

const PolicyCertificate& 
PCI::operator->() const {
  return policyCert;
}

const string&
PCI::getResource() const {
  return policyCert->getResource();
}

string
PCI::hashCode() const {
  return policyCert->hashCode(); 
}

long 
PCI::getCacheInterval() const {
  return policyCert->getCacheTime();
}

const AkentiIssuer& 
PCI::getIssuer() const {
  return policyCert->getIssuer();
}

const PolicyCertificate&
PCI::getPolicyCertificate() const {
  return policyCert;
}

const vector<CertificateAuthorityInfo>& 
PCI::getCertificateAuthorityInfos() const {
  return policyCert->getCertificateAuthorityInfos();
}

const vector<Directory>& 
PCI::getAttributeDirectories() const {
  return policyCert->getAttributeDirectories();
}

int 
PCI::numOfUseCondIssuerGroups() const {
  return policyCert->getUseCondIssuerGroups().size();
}

const UseCondIssuerGroup& 
PCI::getUseCondIssuerGroup(int groupIndex) const {
  if (groupIndex < 0) {
    throw Error("Policy:getUseCondGroup:illegal argument");
  }

  const vector<UseCondIssuerGroup>& groups = policyCert->getUseCondIssuerGroups();

  if (groupIndex >= groups.size()) {
    throw Error("Policy:getUseCondGroup:index out of range");
  }
  
  return groups[groupIndex];
}

const vector<UseCondCertificate>&
PCI::getUCCerts() const {
  return ucCerts;
}

vector<UseCondCertificate> 
PCI::getCriticalUCCerts() const {
  int size = ucCerts.size();
  vector<UseCondCertificate> ret;

  for (int i = 0; i < size; ++i) {
    if (ucCerts[i]->isCritical()) {
      ret.push_back(ucCerts[i]);
    }
  }
  
  return ret;
}

vector<UseCondCertificate> 
PCI::getFlexibleUCCerts() const {
  int size = ucCerts.size();
  vector<UseCondCertificate> ret;

  for (int i = 0; i < size; ++i) {
    if (!ucCerts[i]->isCritical()) {
      ret.push_back(ucCerts[i]);
    }
  }
  
  return ret;
}

void 
PCI::addUCCertificate(const UseCondCertificate& ucCert) {
  ucCerts.push_back(ucCert);
}

string 
PCI::paramString() const {
  string params;

  params += '[';

  if (leaf) {
    params += "LEAF";
    params += ',';
  }

  params += policyCert->paramString();
  
  params += '[';
  for (int i = 0; i < ucCerts.size(); ++i) {
    params += ',';
    params += policyCert->paramString();
  }
  params += ']';
  params += ']';
  return params;
}

string
PCI::toLogString() const {
  return policyCert->toLogString();
}

