#include "PolicyContext.hpp"
#include "Error.hpp"

using namespace Akenti;
using namespace std;

typedef PolicyContextImpl PCI;

PCI::PolicyContextImpl() {
}

PCI::~PolicyContextImpl() {
}

void 
PCI::setResource(const string& resource) {
  this->resource = resource;
}

const string&
PCI::getResource() const {
  return resource;
}

void
PCI::setResourceAltName(const string& resAltName) {
  this->resAltName = resAltName;
}

const string& 
PCI::getResourceAltName() const {
  return resAltName;
}

void
PCI::setAkentiPrincipal(const AkentiPrincipal& ap) {
  this->ap = ap;
}

const AkentiPrincipal&
PCI::getAkentiPrincipal() const {
  return ap;
}

void 
PCI::setSubjectCert(const AkentiX509Certificate& subjectCert) {
  this->subjectCert = subjectCert;
}

const AkentiX509Certificate&
PCI::getSubjectCert() const {
  return subjectCert;
}

void
PCI::setCapabilityCert(const CapabilityCertificate& capCert) {
  this->capCert = capCert;
}

const CapabilityCertificate&
PCI::getCapabilityCert() const {
  return capCert;
}

const vector<Policy>&
PCI::getPolicies() const {
  return policies;
}

void 
PCI::addPolicy(const Policy& policy) {
  policies.push_back(policy);
}

const vector<string>& 
PCI::getUnconditionalActions() const {
  return unconditionalActions;
}

const vector<ConditionalActions>& 
PCI::getConditionalActionsList() const {
  return conditionalActionsList;
}

void 
PCI::setDesiredActions(const set<string>& desiredActions) {
  this->desiredActions = desiredActions;
}

const set<string>&
PCI::getDesiredActions() const {
  return desiredActions;
} 

bool 
PCI::isActionDesired(const string& action) const {
  bool ret = desiredActions.size() == 0; 
  
  if (!ret) {
    set<string>::const_iterator pos = desiredActions.begin();

    for (; !ret && pos != desiredActions.end(); ++pos) {
       ret = *pos == action;
    }
  }
  
  return ret;
}

void
PCI::setRestrictedAttributes(const multimap<string, string>& restrictedAttributes) {
  this->restrictedAttributes = restrictedAttributes;
}

const multimap<string, string>& 
PCI::getRestrictedAttributes() const {
  return restrictedAttributes;
}

bool
PCI::isAttributeDesired(const string& attribute, const string& value) const {
  if (restrictedAttributes.size() == 0) {
    return true;
  }

  multimap<string, string>::const_iterator iter = restrictedAttributes.begin();
  bool ret;

  for (; iter != restrictedAttributes.end(); ++iter) {
    ret = (iter->first == attribute);

    if (ret) {
      break;
    }
  }

  if (!ret) {
    return true;
  }
 
  for (; iter != restrictedAttributes.end(); ++iter) {
    ret = (iter->first == attribute && iter->second == value);

    if (ret) {
      break;
    }
  }

  return ret;
}

void 
PCI::addUnconditionalAction(const string& unconditionalAction) {
  unconditionalActions.push_back(unconditionalAction);
}

void 
PCI::addConditionalActions(const ConditionalActions& condActions) {
  conditionalActionsList.push_back(condActions);
}

void 
PCI::addAkentiX509Cert(const AkentiX509Certificate& cert) {
  x509Certs.push_back(cert);
}

void 
PCI::addAttributeCert(const AttributeCertificate& attrCert) {
  // XXX How about duplicates?
  attrCerts.push_back(attrCert);
}

const vector<AttributeCertificate>&
PCI::getAttributeCerts() const {
  return attrCerts;
}

string
PCI::paramString() const {
  string params;

  params += '[';
  params += "resource";
  params += '=';
  params += resource;
  params += ',';
  params += ap.paramString();
  
  int size = policies.size();

  params += ',';
  params += '[';

  for (int i = 0; i < size - 1; ++i) {
    params += policies[i]->paramString();  
    params += ',';
  }

  if (size) {
     params += policies[size - 1]->paramString();  
     params += ']';
  }

  size = unconditionalActions.size();
  params += ',';
  params += '[';

  for (int i = 0; i < size - 1; ++i) {
    params += unconditionalActions[i];
    params += ',';
  }

  if (size) {
    params += unconditionalActions[size - 1];
  }

  params += ']';

  size = conditionalActionsList.size();
  params += ',';
  params += '[';

  for (int i = 0; i < size - 1; ++i) {
    params += conditionalActionsList[i].paramString();
    params += ',';
  }

  if (size) {
    params += conditionalActionsList[size - 1].paramString();
  }

  params += ']';
  params += ']';
  return params;
}

