#include <sstream>
#include <iostream>
#include "StreamTokenizer.hpp"
#include "Error.hpp"

using std::ostringstream;
using std::istream;

typedef Akenti::StreamTokenizer ST;

ST::StreamTokenizer(istream& source, const string& delimiters, bool retTokens) 
  : source(source), delimiters(delimiters), retTokens(retTokens) {
}

bool 
ST::hasMoreTokens() {
  while (!retTokens && !source.eof() && delimiters.find(source.peek()) != string::npos) {
    source.get();
  }
  
  if (source.eof()) {
    return false;
  }

  /**
    * Funny looking stuff. Don't fight it. :)
    **/
  source.get();
   
  if (source.eof()) {
    return false;
  }

  source.unget();

  return true;
}

string 
ST::nextToken() {
  if (!hasMoreTokens()) {
    throw Error("StringStreamTokenizer::nextToken:no more tokens");
  }

  if (retTokens && delimiters.find(source.peek()) != string::npos) {
    return string(1, (char) source.get());
  }

  string token;

  while (!source.eof() && delimiters.find(source.peek()) == string::npos) {
    char c = source.get();
    
    if (source.eof()) {
      break;
    }

    token += c;
  }
  
  return token;
}

string
ST::paramString() const {
  ostringstream os;

  os << '['
     << "delimiters" 
     << '='
     << '"'
     << delimiters
     << '"'
     << ','
     << "retTokens"  
     << '='
     << retTokens 
     << ']';

  return os.str(); 
} 
