#include <sstream>
#include "StringTokenizer.hpp"
#include "Error.hpp"

using std::ostringstream;

typedef Akenti::StringTokenizer ST;

ST::StringTokenizer(const string& source, const string& delimiters, bool retTokens) 
  : source(source), delimiters(delimiters), currentPosition(0), 
    maxPosition(source.length()), retTokens(retTokens) {

}
	
bool 
ST::hasMoreTokens() {
  while (!retTokens && 
         currentPosition < maxPosition && 
         delimiters.find(source.at(currentPosition)) != string::npos) {

    currentPosition++;
  }
  
  return currentPosition < maxPosition;
}

string 
ST::nextToken() {
  if (!hasMoreTokens()) {
    throw Error("StringTokenizer::nextToken:no more tokens");
  }

  string::size_type startPosition = currentPosition;

  while (currentPosition < maxPosition &&
         delimiters.find(source.at(currentPosition)) == string::npos) {
    currentPosition++;
  }
  
  if (retTokens && 
      startPosition == currentPosition &&
      delimiters.find(source.at(currentPosition) != string::npos)) {
    currentPosition++;
  }

  return source.substr(startPosition, currentPosition - startPosition);
}

int
ST::countTokens() {
  int count = 0;
  int currpos = currentPosition;

  while (currpos < maxPosition) {
    while (!retTokens &&
           currpos < maxPosition &&
           delimiters.find(source.at(currpos)) != string::npos) {
      currpos++;
    }

    if (currpos >= maxPosition) {
      break;
    }

    int start = currpos;

    while (currpos < maxPosition &&
           delimiters.find(source.at(currpos)) == string::npos) {
      currpos++;
    }

    if (retTokens && 
        start == currpos &&
        delimiters.find(source.at(currpos)) != string::npos) {
      currpos++;
    }

    count++;
  }

  return count;
}

string 
ST::paramString() const { 
  ostringstream os;

  os << '['
     << "source"
     << '=' 
     << '"'
     << source
     << '"'
     << ',' 
     << "delimiters"
     << '=' 
     << '"'
     << delimiters
     << '"'
     << ','
     << "retTokens"
     << '=' 
     << retTokens 
     << ','
     << "currentPosition"
     << '=' 
     << currentPosition
     << ','
     << "maxPosition"
     << '='
     << maxPosition 
     << ']';
  return os.str();

}
