#ifndef AK_STRING_TOKENIZER_HPP
#define AK_STRING_TOKENIZER_HPP

#include <string> 

using std::string;

/**
  * This a simple string tokenizer. 
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

namespace Akenti {

class StringTokenizer {

public:

  /**
    * Constructs this object. The default deimiters are any white space 
    * characters. By default the delimiters are not returned as tokens.
    *
    * @param source string to be parsed
    * @param delimiters the set of delimiters used to parse the stream
    * @param retTokens if true the delimiters are returned as a token,
    *                  if false they are not.
    **/
  StringTokenizer(const string& source, 
                  const string& delimiters = "\t\n\r\f ", 
                  bool retTokens = false);
  /**
    * Returns <i>true</i> if the tokenizer has more tokens.
    *
    * @return true if there are more tokens, false otherwise
    */
  bool hasMoreTokens();

  /**
    * Returns the next token.
    *
    * @exception Error if there are no more tokens
    * @return the next token
    */
  string nextToken();

  /**
    * Returns the next token. The new set of delimiters
    * replaces the old one.
    *
    * @param delimiters the new set of delimiters use to parse the stream
    * @exception Error if there are no more tokens
    * @return the next token
    * @see nextToken()
    */
  string nextToken(const string& delims) {
    this->delimiters = delims;
    return nextToken();
  }

  /**
    */
  int countTokens();

  /**
    * Prints the data members for <i>StringTokenizer</i> using the format
    * [nameOfDataMember="valueOfDataMember",...]
    *
    * @return the string format as shown above
    */
  string paramString() const;

private:

  string source;           /* string to be parsed */

  string delimiters;       /* set of delimiters */

  bool 
    retTokens;             /* set to true if the delimiters are to be returned */

  string::size_type 
    currentPosition;       /* points to the current position */

  string::size_type 
    maxPosition;           /* maximum position */

};

} // end of namespace Akenti

#endif
