#include <stdio.h>
#include <sstream>
#include <iostream>
#include "UtcTime.hpp"
#include "Error.hpp" 
#include "Exception.hpp" 

using std::ostringstream;
using std::istream;
using std::ostream;
using std::string;
using Akenti::UtcTime;

UtcTime::UtcTime(const string& utcTime, const string format) {
  struct tm ts;

  // do not remove this
  std::memset(&ts, 0, sizeof(ts));

  if(utcTime.find_last_of("Z") != utcTime.size() -1){
    throw Exception("UtcTime::Constructor:date must contain a 'Z'");
  } else if (format == "YYMMDDHHMMSSZ") {
    sscanf(
    (char*) utcTime.data(),
    "%02d%02d%02d%02d%02d%02dZ",
     &ts.tm_year,
     &ts.tm_mon,
     &ts.tm_mday,
     &ts.tm_hour,
     &ts.tm_min,
     &ts.tm_sec
    );
  } else if (format == "YYYY-MM-DDTHH:MM:SSZ") {
    sscanf(
    (char*) utcTime.data(),
    "%04d-%02d-%02dT%02d:%02d:%02dZ",
     &ts.tm_year,
     &ts.tm_mon,
     &ts.tm_mday,
     &ts.tm_hour,
     &ts.tm_min,
     &ts.tm_sec
    );

    ts.tm_year = ts.tm_year % 100;
  } else {
    throw Exception("UtcTime::Constructor:bad time format");
  }

  // ts.tm_year is number of years since 1900
  if (ts.tm_year <= 49 ) {  ts.tm_year += 100; }

  // ts.tm_mon range from 0-11
  ts.tm_mon -= 1;

  if ((dtime = mktime(&ts)) == -1) {
    throw Error("UtcTime::Constructor:mktime failed");
  }

  // mktime adds 1 hour if daylight savings is on.
  dtime -= ts.tm_isdst * 60 * 60;
}

UtcTime::UtcTime(int offset) {
  dtime = time(NULL) + offset;

  struct tm *ts = gmtime(&dtime);

  if ((dtime = mktime(gmtime(&dtime))) == -1) {
    throw Error("UtcTime::Constructor:mktime failed");
  }

  // mktime adds 1 hour if daylight savings is on.
  dtime -= ts->tm_isdst * 60 * 60;
}

UtcTime::~UtcTime() {
}

string 
UtcTime::toString(const string format) const {
  if(format == "YYMMDDHHMMSSZ"){
    char buf[13];
    
    toCharArray(buf,format);
    string rep(buf, 13);

    return rep;
  }else if(format == "YYYY-MM-DDTHH:MM:SSZ"){
    char buf[20];
    
    toCharArray(buf,format);
    string rep(buf, 20);

    return rep;
  } else {
    throw Error("UtcTime::bad time format");
  }

}

void 
UtcTime::toCharArray(char buffer[], const string format) const {
  struct tm* ts = localtime(&dtime);

  if (ts == NULL) {
    throw Error("UtcTime::toCharArray:local time failed");
  }

  if(format == "YYMMDDHHMMSSZ"){
    sprintf(
     buffer,
     "%02d%02d%02d%02d%02d%02dZ",
     ts->tm_year%100,
     ts->tm_mon+1,
     ts->tm_mday,
     ts->tm_hour,
     ts->tm_min,
     ts->tm_sec
    );
  }else if(format == "YYYY-MM-DDTHH:MM:SSZ"){
    sprintf(
     buffer,
     "%04d-%02d-%02dT%02d:%02d:%02dZ",
     ts->tm_year+1900,
     ts->tm_mon+1,
     ts->tm_mday,
     ts->tm_hour,
     ts->tm_min,
     ts->tm_sec
    );
  } else {
    throw Error("UtcTime::bad time format");
  }
}

UtcTime 
UtcTime::operator+(int offset) const {
  UtcTime ret;

  ret.dtime = dtime + offset;
  return ret;
}

UtcTime&
UtcTime::operator+=(int offset) {
  dtime += offset;
  return *this;
}

bool 
UtcTime::operator==(const UtcTime &utcTime) const {
  return dtime == utcTime.dtime;
}

bool 
UtcTime::operator<(const UtcTime &utcTime) const {
  return dtime < utcTime.dtime;
}

bool 
UtcTime::operator<=(const UtcTime &utcTime) const {
  return dtime <= utcTime.dtime;
}

bool 
UtcTime::operator>(const UtcTime &utcTime) const {
  return dtime > utcTime.dtime;
}

bool 
UtcTime::operator>=(const UtcTime &utcTime) const {
  return dtime >= utcTime.dtime;
}

string 
UtcTime::paramString() const {
  return toString();
}
 
ostream&
operator<<(ostream& os, const Akenti::UtcTime& utcTime) {
  return os << utcTime.toString();
}

istream&
operator>>(istream& is, Akenti::UtcTime& utcTime) {
  string  rep;
  
  is >> rep;
  utcTime = UtcTime(rep);
  return is;
}
