#include <openssl/x509.h>
#include "OpenSSLCertificateRequestGeneratorImpl.hpp"
#include "CertificateRequest.hpp"
#include "PrivateKey.hpp"
#include "SecurityException.hpp"
#include "OpenSSLCertificateRequestImpl.hpp"
#include "OpenSSLPublicKeyImpl.hpp"
#include "OpenSSLPrivateKeyImpl.hpp"
#include "OpenSSLSignatureImpl.hpp"
#include "OpenSSLUtil.hpp"

typedef Akenti::OpenSSLCertificateRequestGeneratorImpl OSCRGPI;

OSCRGPI::OpenSSLCertificateRequestGeneratorImpl() {
}

OSCRGPI::~OpenSSLCertificateRequestGeneratorImpl() {
}

void 
OSCRGPI::setSubject(const DistinguishedName& subj) {
  this->subject = subj;
}

void 
OSCRGPI::setPublicKey(const PublicKey& subjKey) {
  this->subjectKey = subjKey;
}

void 
OSCRGPI::setVersionNumber(long vn) {
  this->versionNumber = vn;
}

Akenti::CertificateRequest 
OSCRGPI::generate(const string& sigalg, const PrivateKey& privateKey) {
  X509_REQ* delegate = X509_REQ_new();

  CertificateRequest       
    request(new OpenSSLCertificateRequestImpl(delegate));

  OpenSSLPublicKeyImpl* 
    publicKeyImpl = dynamic_cast<OpenSSLPublicKeyImpl*> (subjectKey.get());

  if (!publicKeyImpl) {
    throw Error("OpenSSLCertificateFactoryImpl:unrecognized public key");
  }

  X509_REQ_set_pubkey(delegate, publicKeyImpl->getEVP_PKEY());

  X509_NAME *name = X509_REQ_get_subject_name(delegate);
  
  OpenSSLUtil::populateX509Name(name, subject);
  X509_REQ_set_version(delegate, versionNumber);

  OpenSSLPrivateKeyImpl* 
    privateKeyImpl = dynamic_cast<OpenSSLPrivateKeyImpl*> (privateKey.get());

  if (!privateKeyImpl) {
    throw Error("OpenSSLCertificateFactoryImpl:unrecognized private key");
  }

  const EVP_MD *md = OpenSSLSignatureImpl::getEVP_MD(sigalg);

  if (X509_REQ_sign(delegate, privateKeyImpl->getEVP_PKEY(), md) <= 0) {
    throw SecurityException("OpenSSLCertificateFactoryImpl:signature failed");
  }

  return request;
}

string 
OSCRGPI::paramString() const {
  string param;

  param += "[";
  param += "subject=";
  param += subject.paramString(); 
  param += ","; 
  param += "subject pubkey=" ;
  param += subjectKey->paramString();
  param += ","; 
  param += "version=";
  param += versionNumber;
  param += "]";
  return param;
}


