#include <sstream>
#include "OpenSSLX509RevokedCertificateImpl.hpp"
#include "Certificate.hpp"

using std::ostringstream;

typedef Akenti::OpenSSLX509RevokedCertificateImpl OSRCI;

OSRCI::OpenSSLX509RevokedCertificateImpl(long serialNumber, const UtcTime& date) 
  : serialno(serialNumber), revokedDate(date) {
}

OSRCI::OpenSSLX509RevokedCertificateImpl(const Certificate &cert, const UtcTime& date) 
  : serialno(cert->getSerialNumber()), revokedDate(date) {
}

OSRCI::OpenSSLX509RevokedCertificateImpl(long serialNumber, 
                                         const UtcTime& date, 
                                         const std::vector<V3Extension>& v3extensions) 
  : serialno(serialNumber), revokedDate(date), v3extensions(v3extensions) {
}

OSRCI::~OpenSSLX509RevokedCertificateImpl() {
}

long
OSRCI::getCertificateNumber() const {
  return serialno;
}

Akenti::UtcTime
OSRCI::getRevocationDate() const {
  return revokedDate;
}

std::vector<Akenti::V3Extension>
OSRCI::getExtensions() const {
  return v3extensions;
}

string
OSRCI::paramString() const {
  ostringstream os;
  ostringstream os1;
  std::vector<V3Extension> vec = getExtensions(); 

  os << '['
     << "serialno" 
     << '='
     << getCertificateNumber() 
     << ','
     << "revokedDate"
     << '='
     << '"'
     << getRevocationDate().paramString() 
     << '"'
     << ','
     << '['
     << "v3extensions"
     << ':'
     << '"';

  for(int i = 0; i < vec.size(); i++) {
    os << vec[i].paramString() << ',';
  }

  os << '"'
     << ']'
     << ']';
  return os.str();
}
