#ifndef AK_CERTIFICATE_CHAIN_IMPL_HPP
#define AK_CERTIFICATE_CHAIN_IMPL_HPP

#include <string>
#include <vector>

using std::string;
using std::vector;
using std::pair;

namespace Akenti {

class DistinguishedName;
class Certificate;

/**
  * This abstract class defines an interface to a generic chain of certificates.
  *
  * @see DistinguishedName
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

class CertificateChainImpl {

public:

  /**
    * Deletes this object.
    */ 
  virtual ~CertificateChainImpl() {
  }

  /**
    * Returns the vector of authority certificates  
    */
  virtual vector<Certificate> getCACertificates() const = 0;

  /**
    * Ignore certificates that pattern matches
    * portion of distinguished names of the subject from the chain
    * These ignored certificates will not be considered
    * when selecting a subject certificate
    *
    * NOTE: only subject certificates can be ignored.
    *       authority certificates cannot be ignored.
    */
  virtual void ignoreCertificates(const vector < pair<string, string> >& pairs) = 0;

  /**
    * Returns the subject's certificate.
    */
  virtual Certificate getSubjectCertificate() const = 0;

  /**
    * Returns guarantor of the subject. 
    */
  virtual DistinguishedName getGuarantor() const = 0;

  /**
    * Returns the subject.
    */
  virtual DistinguishedName getPrincipal() const = 0;

  /**
    * Returns the string of items related to this certificate chain
    * for correctness check.
    */
  virtual string paramString() const = 0;

protected:

  /**
    * Constructs this object.
    */
  CertificateChainImpl() {
  }

};

} // end of namespace Akenti

#endif
