#ifndef AK_V3EXTENSION_HPP
#define AK_V3EXTENSION_HPP

#include <string>

using std::string;

namespace Akenti {

/**
  * This class implements the extended functionalities of Version 3
  * of X509 certificates.  X.509 certificate extensions consist of
  * an identification and value an a boolean value, <i> critical</i>. If
  * <i>critical</i> is true the extension must be evaluated when verifying
  * the certificate. If it is false, it is left up to the application whether
  * to evaluate the extension.
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

class V3Extension {

public:

  /**
    * Constructs this object.
    */ 
  V3Extension(int extnID, bool critical, const string& value);
 
  /**
    * Deletes this object.
    */
  ~V3Extension();
 
  /**
    * Returns extensionID <i> extnID</i>
    *
    */
  int getExtensionID() const ;

  /**
    * Returns true or false to indicate if the extension must be
    * used or not.
    */ 
  bool isCritical() const;

  /**
    * Returns value of the extension.
    */
  const string& getValue() const;

  /**
    *
    */
  string paramString() const;

private:

  int extnID;

  bool critical;

  string value;

};

} // end of namespace Akenti

#endif
